# ecogVIS
# Timeseries visualizer for Electrocorticography (ECoG) signals stored in NWB files.
# Developed by:
# Luiz Tauffer, Ben Dichter
# https://github.com/catalystneuro/ecogVIS
#
import sys
import os
import numpy as np
import datetime
import glob
import yaml
from pathlib import Path

from PyQt5 import QtCore, QtGui
from PyQt5.QtCore import QCoreApplication
from PyQt5.QtWidgets import (QWidget, QLabel, QLineEdit, QMessageBox, QHBoxLayout,
                             QApplication, QPushButton, QVBoxLayout, QGroupBox,
                             QGridLayout, QComboBox, QInputDialog, QFileDialog,
                             QMainWindow, QAction, QStackedLayout)
import pyqtgraph as pg

from ecogvis.functions.subFunctions import TimeSeriesPlotter
from ecogvis.functions.misc_dialogs import (CustomIntervalDialog, SelectChannelsDialog,
                                            SpectralChoiceDialog, NoHighGammaDialog,
                                            NoPreprocessedDialog, NoTrialsDialog,
                                            ExitDialog, HighGammaDialog, LoadHTKDialog,
                                            PeriodogramGridDialog, NoSpectrumDialog,
                                            PreprocessingDialog, NoRawDialog,
                                            NoAudioDialog, ExistIntervalsDialog,
                                            ShowSurveyDialog,
                                            ShowElectrodesDialog, ShowTranscriptionDialog)
from ecogvis.functions.audio_event_detection import AudioEventDetection
from ecogvis.functions.event_related_potential import ERPDialog
from ecogvis.functions.save_to_nwb import SaveToNWBDialog
from ecogvis.functions.nwb_copy_file import nwb_copy_file
from ecogvis.functions.survey_data import add_survey_data
from ecogvis.functions.transcription_data import add_transcription_data
from ecogvis.functions.htk_to_nwb.chang2nwb import chang2nwb


annotationAdd_ = False
annotationDel_ = False
annotationColor_ = 'red'
intervalAdd_ = False
intervalDel_ = False
intervalType_ = 'invalid'
intervalsDict_ = {'invalid': {'type': 'invalid',
                              'session': '',
                              'color': 'red',
                              'counts': 0}}


class Application(QMainWindow):
    """
    Main EcogVIS window.

    Parameters
    ----------
    source_path : str or path
        Path to the source of data, nwb file or htk directory.
    """
    keyPressed = QtCore.pyqtSignal(QtCore.QEvent)

    def __init__(self, source_path=None, metafile=None, session=None):
        super().__init__()
        # Enable anti-aliasing for prettier plots
        pg.setConfigOptions(antialias=True)

        self.centralwidget = QWidget()
        self.setCentralWidget(self.centralwidget)
        self.resize(1000, 600)
        self.setWindowTitle('ecogVIS')

        # Check source of data
        if not os.path.exists(source_path):
            source_path = '.'
        self.source_path = Path(source_path).absolute()

        # Check metadata file
        self.metadata = None
        if metafile is not None:
            with open(metafile) as f:
                self.metadata = yaml.safe_load(f)

        self.error = None
        self.keyPressed.connect(self.on_key)
        self.active_mode = 'default'

        self.init_gui()
        self.show()

        # opens dialog for session name
        if session is None:
            text = 'Enter session name:\n(e.g. your_name)'
            self.enter_sesssion_name_dialog = QInputDialog()
            uinp, ok = self.enter_sesssion_name_dialog.getText(None, 'Choose session', text)
            if ok:
                self.current_session = uinp
            else:
                self.current_session = 'default'
        else:
            self.current_session = session

        # Run the main plotting function
        if self.source_path.is_file():
            self.model = TimeSeriesPlotter(self)

    def closeEvent(self, event):
        """Before exiting, checks if there are any unsaved changes and inform the user."""
        w = ExitDialog(self)
        if w.value == -1:  # just exit
            self.model.close_nwbfile()
            event.accept()
        elif w.value == 1:  # save and exit
            self.AnnotationSave()
            self.IntervalSave()
            self.model.close_nwbfile()
            event.accept()
        elif w.value == 0:  # ignore
            event.ignore()

    def log_error(self, error):
        """Error logging"""
        pwd = os.getcwd()
        folder = os.path.join(pwd, 'error_log')
        if not os.path.exists(folder):
            os.mkdir(folder)
        with open(os.path.join(folder, 'error.log'), 'a') as file:
            file.write(datetime.datetime.today().strftime('%Y-%m-%d  %H:%M:%S') + ' ' + error + '\n')
        file.close()

    def keyPressEvent(self, event):
        super(Application, self).keyPressEvent(event)
        self.keyPressed.emit(event)

    def on_key(self, event):
        """Actions to be taken when user presses keys."""
        if event.key() == QtCore.Qt.Key_Up:
            self.model.channel_Scroll_Up('page')
        elif event.key() == QtCore.Qt.Key_PageUp:
            self.model.channel_Scroll_Up('page')
        elif event.key() == QtCore.Qt.Key_Down:
            self.model.channel_Scroll_Down('page')
        elif event.key() == QtCore.Qt.Key_PageDown:
            self.model.channel_Scroll_Down('page')
        elif event.key() == QtCore.Qt.Key_Left:
            self.model.time_scroll(scroll=-1 / 3)
        elif event.key() == QtCore.Qt.Key_Right:
            self.model.time_scroll(scroll=1 / 3)
        event.accept()

    def init_gui(self):
        """Initiates GUI elements."""
        mainMenu = self.menuBar()
        # File menu
        fileMenu = mainMenu.addMenu('File')
        # Adding actions to file menu
        change_session_file = QAction('Change Session', self)
        fileMenu.addAction(change_session_file)
        change_session_file.triggered.connect(self.change_session)
        open_tools_menu = fileMenu.addMenu('Open from')
        action_open_nwb = QAction('NWB', self)
        open_tools_menu.addAction(action_open_nwb)
        action_open_nwb.triggered.connect(lambda: self.open_another_file(None))
        action_open_htk = QAction('HTK dir', self)
        open_tools_menu.addAction(action_open_htk)
        action_open_htk.triggered.connect(self.open_htk_dir)

        action_save_new_file = QAction('Save to NWB', self)
        fileMenu.addAction(action_save_new_file)
        action_save_new_file.triggered.connect(self.save_file)

        toolsMenu = mainMenu.addMenu('Tools')
        action_load_annotations = QAction('Load Annotations', self)
        toolsMenu.addAction(action_load_annotations)
        action_load_annotations.triggered.connect(self.load_annotations)
        action_load_intervals = QAction('Load Intervals', self)
        toolsMenu.addAction(action_load_intervals)
        action_load_intervals.triggered.connect(self.load_intervals)
        channels_tools_menu = toolsMenu.addMenu('Bad Channels')
        action_add_badchannel = QAction('Add Bad Channel', self)
        channels_tools_menu.addAction(action_add_badchannel)
        action_add_badchannel.triggered.connect(self.add_badchannel)
        action_del_badchannel = QAction('Del Bad Channel', self)
        channels_tools_menu.addAction(action_del_badchannel)
        action_del_badchannel.triggered.connect(self.del_badchannel)
        action_spectral_decomposition = QAction('Spectral Decomposition', self)
        toolsMenu.addAction(action_spectral_decomposition)
        action_spectral_decomposition.triggered.connect(self.spectral_decomposition)
        action_erp = QAction('ERP', self)
        toolsMenu.addAction(action_erp)
        action_erp.triggered.connect(self.event_related_potential)
        action_event_detection = QAction('CV Event Detection', self)
        toolsMenu.addAction(action_event_detection)
        action_event_detection.triggered.connect(self.audio_event_detection)

        survey_tools_menu = toolsMenu.addMenu('Survey')
        self.action_add_survey = QAction('Add Survey', self)
        survey_tools_menu.addAction(self.action_add_survey)
        self.action_add_survey.triggered.connect(self.add_survey)
        self.action_vis_survey = QAction('Visualize Survey', self)
        survey_tools_menu.addAction(self.action_vis_survey)
        self.action_vis_survey.setEnabled(False)
        self.action_vis_survey.triggered.connect(self.visualize_survey)

        transcription_tools_menu = toolsMenu.addMenu('Transcription Data')
        self.transcriptionadd_tools_menu = transcription_tools_menu.addMenu('Add Transcription Data')
        action_add_timitsounds = QAction('TimitSounds', self)
        self.transcriptionadd_tools_menu.addAction(action_add_timitsounds)
        action_add_timitsounds.triggered.connect(self.add_transcription_timitsounds)
        action_add_textgrid = QAction('TextGrid', self)
        self.transcriptionadd_tools_menu.addAction(action_add_textgrid)
        action_add_textgrid.triggered.connect(self.add_transcription_textgrid)
        action_add_mocha = QAction('Mocha', self)
        self.transcriptionadd_tools_menu.addAction(action_add_mocha)
        action_add_mocha.triggered.connect(self.add_transcription_mocha)
        self.action_vis_transcription = QAction('Visualize Transcription Data', self)
        transcription_tools_menu.addAction(self.action_vis_transcription)
        self.action_vis_transcription.triggered.connect(self.visualize_transcription)

        self.action_vis_electrodes = QAction('Visualize Electrodes Tables', self)
        toolsMenu.addAction(self.action_vis_electrodes)
        self.action_vis_electrodes.triggered.connect(self.visualize_electrodes)

        helpMenu = mainMenu.addMenu('Help')
        action_about = QAction('About', self)
        helpMenu.addAction(action_about)
        action_about.triggered.connect(self.about)

        # Buttons and controls vertical box layout -----------------------------
        # Block change buttons
        self.qlabelBlocks = QLabel('Move Block')
        self.pushBlock_0 = QPushButton('<')
        self.pushBlock_0.clicked.connect(lambda: self.change_block(-1))
        self.pushBlock_1 = QPushButton('>')
        self.pushBlock_1.clicked.connect(lambda: self.change_block(1))
        self.pushBlock_0.setEnabled(False)
        self.pushBlock_1.setEnabled(False)
        grid0 = QGridLayout()
        grid0.addWidget(self.pushBlock_0, 0, 0, 1, 1)
        grid0.addWidget(self.qlabelBlocks, 0, 1, 1, 3)
        grid0.addWidget(self.pushBlock_1, 0, 4, 1, 1)

        panel1 = QGroupBox('Panel')
        panel1.setFixedWidth(250)

        # Annotation buttons
        qlabelAnnotations = QLabel('Annotation:')
        self.combo1 = QComboBox()
        self.combo1.addItem('red')
        self.combo1.addItem('green')
        self.combo1.addItem('blue')
        self.combo1.addItem('yellow')
        self.combo1.activated.connect(self.AnnotationColor)
        self.push1_1 = QPushButton('Add')
        self.push1_1.clicked.connect(self.AnnotationAdd)
        self.push1_1.setCheckable(True)
        self.push1_2 = QPushButton('Del')
        self.push1_2.clicked.connect(self.AnnotationDel)
        self.push1_2.setCheckable(True)
        self.push1_3 = QPushButton('Save')
        self.push1_3.clicked.connect(self.AnnotationSave)

        # Custom intervals buttons
        qlabelIntervals = QLabel('Intervals:')
        self.combo2 = QComboBox()
        self.combo2.addItem('invalid')
        self.combo2.addItem('add custom')
        self.combo2.activated.connect(self.IntervalType)
        self.push2_1 = QPushButton('Add')
        self.push2_1.clicked.connect(self.IntervalAdd)
        self.push2_1.setCheckable(True)
        self.push2_2 = QPushButton('Del')
        self.push2_2.clicked.connect(self.IntervalDel)
        self.push2_2.setCheckable(True)
        self.push2_3 = QPushButton('Save')
        self.push2_3.clicked.connect(self.IntervalSave)

        # Bad channels buttons
        qlabelBadChannels = QLabel('Bad channels:')
        self.push3_1 = QPushButton('Add')
        self.push3_1.clicked.connect(self.add_badchannel)
        self.push3_2 = QPushButton('Del')
        self.push3_2.clicked.connect(self.del_badchannel)

        # Get channel by brain region
        self.push4_0 = QPushButton('Select regions')
        self.push4_0.clicked.connect(self.ChannelSelect)
        # Periodogram
        self.push5_0 = QPushButton('Periodogram')
        self.push5_0.clicked.connect(self.PeriodogramSelect)
        # One-click preprocess signals
        self.push6_0 = QPushButton('Preprocess')
        self.push6_0.setCheckable(False)
        self.push6_0.clicked.connect(self.Preprocess)
        # One-click calculate High Gamma
        self.push7_0 = QPushButton('High Gamma')
        self.push7_0.setCheckable(False)
        self.push7_0.clicked.connect(self.CalcHighGamma)

        # Buttons layout
        grid1 = QGridLayout()
        grid1.addWidget(qlabelAnnotations, 0, 0, 1, 3)
        grid1.addWidget(self.combo1, 0, 3, 1, 3)
        grid1.addWidget(self.push1_1, 1, 0, 1, 2)
        grid1.addWidget(self.push1_2, 1, 2, 1, 2)
        grid1.addWidget(self.push1_3, 1, 4, 1, 2)
        grid1.addWidget(qlabelIntervals, 2, 0, 1, 3)
        grid1.addWidget(self.combo2, 2, 3, 1, 3)
        grid1.addWidget(self.push2_1, 3, 0, 1, 2)
        grid1.addWidget(self.push2_2, 3, 2, 1, 2)
        grid1.addWidget(self.push2_3, 3, 4, 1, 2)
        grid1.addWidget(qlabelBadChannels, 4, 0, 1, 6)
        grid1.addWidget(self.push3_1, 5, 0, 1, 2)
        grid1.addWidget(self.push3_2, 5, 2, 1, 2)
        grid1.addWidget(self.push4_0, 6, 0, 1, 3)
        grid1.addWidget(self.push5_0, 6, 3, 1, 3)
        grid1.addWidget(self.push6_0, 7, 0, 1, 3)
        grid1.addWidget(self.push7_0, 7, 3, 1, 3)
        panel1.setLayout(grid1)

        # Traces panel ---------------------------------------------------------
        panel2 = QGroupBox('Traces')
        panel2.setFixedWidth(250)
        qlabelSignal = QLabel('Signal:')
        self.combo3 = QComboBox()
        self.combo3.addItem('raw')
        self.combo3.addItem('preprocessed')
        self.combo3.addItem('high gamma')
        self.combo3.activated.connect(self.voltage_time_series)
        qlabelStimuli = QLabel('Stimuli:')
        self.combo4 = QComboBox()
        self.combo4.activated.connect(self.choose_stim)

        grid2 = QGridLayout()
        grid2.addWidget(qlabelSignal, 0, 0, 1, 1)
        grid2.addWidget(self.combo3, 0, 1, 1, 1)
        grid2.addWidget(qlabelStimuli, 1, 0, 1, 1)
        grid2.addWidget(self.combo4, 1, 1, 1, 1)
        grid2.setAlignment(QtCore.Qt.AlignTop)
        panel2.setLayout(grid2)

        # Plot controls panel --------------------------------------------------
        qlabel1 = QLabel('Top')
        qlabel2 = QLabel('Bottom')
        qlabel3 = QLabel('Interval \nstart(s)')
        qlabel4 = QLabel('Time \nspan')
        qlabel5 = QLabel('Vertical\nScale')
        self.qline0 = QLineEdit('16')
        self.qline1 = QLineEdit('1')
        self.qline2 = QLineEdit('0.01')
        self.qline3 = QLineEdit('2')
        self.qline4 = QLineEdit('1')

        self.qline0.returnPressed.connect(self.channelDisplayed)
        self.qline1.returnPressed.connect(self.channelDisplayed)
        self.qline2.returnPressed.connect(self.interval_start)
        self.qline3.returnPressed.connect(self.time_window_size)
        self.qline4.returnPressed.connect(self.verticalScale)

        self.pushbtn1_1 = QPushButton('^')
        self.pushbtn1_1.clicked.connect(self.scroll_up)
        self.pushbtn1_2 = QPushButton('^^')
        self.pushbtn1_2.clicked.connect(self.scroll_up_page)
        self.pushbtn2_1 = QPushButton('v')
        self.pushbtn2_1.clicked.connect(self.scroll_down)
        self.pushbtn2_2 = QPushButton('vv')
        self.pushbtn2_2.clicked.connect(self.scroll_down_page)

        self.pushbtn3 = QPushButton('<<')
        self.pushbtn3.clicked.connect(self.page_backward)
        self.pushbtn4 = QPushButton('<')
        self.pushbtn4.clicked.connect(self.scroll_backward)
        self.pushbtn5 = QPushButton('>>')
        self.pushbtn5.clicked.connect(self.page_forward)
        self.pushbtn6 = QPushButton('>')
        self.pushbtn6.clicked.connect(self.scroll_forward)
        self.pushbtn7 = QPushButton('*2')
        self.pushbtn7.clicked.connect(self.time_window_enlarge)
        self.pushbtn8 = QPushButton('/2')
        self.pushbtn8.clicked.connect(self.time_window_reduce)
        self.pushbtn9 = QPushButton('*2')
        self.pushbtn9.clicked.connect(self.verticalScaleIncrease)
        self.pushbtn10 = QPushButton('/2')
        self.pushbtn10.clicked.connect(self.verticalScaleDecrease)

        form_2 = QGridLayout()
        form_2.addWidget(qlabel1, 0, 0)
        form_2.addWidget(self.qline0, 0, 1)
        form_2.addWidget(self.pushbtn1_1, 0, 2)
        form_2.addWidget(self.pushbtn1_2, 0, 3)
        form_2.addWidget(qlabel2, 1, 0)
        form_2.addWidget(self.qline1, 1, 1)
        form_2.addWidget(self.pushbtn2_1, 1, 2)
        form_2.addWidget(self.pushbtn2_2, 1, 3)

        form_2.addWidget(qlabel3, 2, 0, 1, 2)
        form_2.addWidget(self.qline2, 2, 2, 1, 2)
        form_2.addWidget(self.pushbtn3, 3, 0)
        form_2.addWidget(self.pushbtn4, 3, 1)
        form_2.addWidget(self.pushbtn6, 3, 2)
        form_2.addWidget(self.pushbtn5, 3, 3)
        form_2.addWidget(qlabel4, 4, 0)
        form_2.addWidget(self.qline3, 4, 1)
        form_2.addWidget(self.pushbtn7, 4, 2)
        form_2.addWidget(self.pushbtn8, 4, 3)
        form_2.addWidget(qlabel5, 5, 0)
        form_2.addWidget(self.qline4, 5, 1)
        form_2.addWidget(self.pushbtn9, 5, 2)
        form_2.addWidget(self.pushbtn10, 5, 3)

        panel3 = QGroupBox('Plot Controls')
        panel3.setFixedWidth(250)
        panel3.setLayout(form_2)

        self.vbox1 = QVBoxLayout()
        self.vbox1.addLayout(grid0)
        self.vbox1.addWidget(panel1)
        self.vbox1.addWidget(panel2)
        self.vbox1.addWidget(panel3)
        self.vbox1.addStretch()

        # Time series plots Vertical Box ---------------------------------------
        vb = CustomViewBox(self)
        self.win1 = pg.PlotWidget(viewBox=vb)   # middle signals plot
        self.win2 = pg.PlotWidget()               # upper horizontal bar
        self.win3 = pg.PlotWidget()               # lower audio plot
        self.win1.setBackground(self.palette().color(QtGui.QPalette.Background))
        self.win2.setBackground(self.palette().color(QtGui.QPalette.Background))
        self.win3.setBackground(self.palette().color(QtGui.QPalette.Background))
        self.win1.setMouseEnabled(x=False, y=False)
        self.win2.setMouseEnabled(x=False, y=False)
        self.win3.setMouseEnabled(x=False, y=False)
        self.win1.hideButtons()
        self.win2.hideButtons()
        self.win3.hideButtons()
        # self.win2.hideAxis('left')
        self.win2.hideAxis('bottom')
        # self.win3.hideAxis('left')
        self.win3.hideAxis('bottom')

        form_3 = QGridLayout()
        form_3.setSpacing(0.0)
        form_3.setRowStretch(0, 1)
        form_3.setRowStretch(1, 8)
        form_3.setRowStretch(2, 1)
        form_3.addWidget(self.win2)
        form_3.addWidget(self.win1)
        form_3.addWidget(self.win3)

        self.groupbox1 = QGroupBox('Signals')
        self.groupbox1.setLayout(form_3)

        self.vbox2 = QStackedLayout()
        self.vbox2.addWidget(self.groupbox1)
        self.vbox2.setCurrentIndex(0)

        # Creates a horizontal box layout and populate it with panels ----------
        self.hbox = QHBoxLayout(self.centralwidget)
        self.hbox.addLayout(self.vbox1)    # add panels first
        self.hbox.addLayout(self.vbox2)    # add plots second

    def change_block(self, move):
        """Move between Block files for the same subject."""
        fpath = os.path.split(os.path.abspath(self.source_path))[0]  # current directory
        fname = os.path.split(os.path.abspath(self.source_path))[1]  # current file
        pre, _ = fname.split('_B')
        flist = glob.glob(fpath + '/' + pre + '_B*.nwb')
        curr_ind = flist.index(self.source_path)
        if curr_ind + move == len(flist):
            new_file = flist[0]
        else:
            new_file = flist[curr_ind + move]
        self.open_another_file(filename=new_file)

    def open_file(self):
        """Opens initial file."""
        filename, _ = QFileDialog.getOpenFileName(None, 'Open file', '', "(*.nwb)")
        return filename

    def open_another_file(self, filename=None):
        """Opens another NWB file."""
        if filename is None:  # Opens new file dialog
            filename, _ = QFileDialog.getOpenFileName(None, 'Open file', '', "(*.nwb)")
        if os.path.isfile(filename):
            if hasattr(self, 'model'):
                self.model.io.close()
            self.source_path = Path(filename)
            # Reset file specific variables on GUI
            self.combo3.setCurrentIndex(self.combo3.findText('raw'))
            self.combo4.clear()
            self.push5_0.setEnabled(True)
            self.push6_0.setEnabled(True)
            self.push7_0.setEnabled(True)
            self.qline0.setText('16')
            self.qline1.setText('1')
            self.qline2.setText('0.01')
            self.qline3.setText('2')
            self.qline4.setText('1')
            self.win1.clear()
            self.win2.clear()
            self.win3.clear()
            # Rebuild the model
            self.model = TimeSeriesPlotter(self)

    def open_htk_dir(self):
        """
        Dialog for user selection of directory containing block of HTK files.
        Assembles a nwbfile object (in memory) from htk files.
        """
        w = LoadHTKDialog(parent=self)
        if w.value == 1 and w.htk_config:
            if hasattr(self, 'model'):
                self.model.io.close()
            htk_source_path = Path(w.htk_config['ecephys_path'])
            self.metadata = w.htk_config['metadata']
            # Run conversion
            _, source_path, _, _ = chang2nwb(
                blockpath=htk_source_path,
                save_to_file=True,
                htk_config=w.htk_config,
            )
            self.source_path = source_path.absolute()
            # Reset file specific variables on GUI
            self.combo3.setCurrentIndex(self.combo3.findText('raw'))
            self.combo4.clear()
            self.push5_0.setEnabled(True)
            self.push6_0.setEnabled(True)
            self.push7_0.setEnabled(True)
            self.qline0.setText('16')
            self.qline1.setText('1')
            self.qline2.setText('0.01')
            self.qline3.setText('2')
            self.qline4.setText('1')
            self.win1.clear()
            self.win2.clear()
            self.win3.clear()
            # Rebuild the model
            self.model = TimeSeriesPlotter(par=self)

    def save_file(self):
        """
        Saves chosen data and info to new NWB file.
        The dialog assembles a dictionary with fields to be copied from the current
        nwb file to the new file. This dictionary is passed to nwb_copy_file()
        function.
        """
        save_dialog = SaveToNWBDialog(parent=self)
        if save_dialog.value:
            print('Copying content to new nwb file, please wait...')
            nwb_copy_file(
                old_file=self.model.nwb,
                new_file=save_dialog.newfile,
                cp_objs=save_dialog.cp_objs,
                save_to_file=True
            )
            print('File successfully copied!')

    def change_session(self):
        """Changes session name."""
        text = 'Current session is: ' + self.current_session + '\nEnter new session name:'
        uinp, ok = QInputDialog.getText(None, 'Change session', text)
        if ok:
            self.current_session = uinp
            fname = os.path.split(os.path.abspath(self.source_path))[1]  # file
            self.setWindowTitle('ecogVIS - ' + fname + ' - ' + self.current_session)

    def load_annotations(self):
        """Loads annotations from file, calls function to paint them."""
        fname, aux = QFileDialog.getOpenFileName(self, 'Open file', '', "(*.csv)")
        if fname != '':
            self.model.AnnotationLoad(fname=fname)

    def load_intervals(self):
        """Loads intervals from file, calls function to paint them."""
        fname, aux = QFileDialog.getOpenFileName(self, 'Open file', '', "(*.csv)")
        if fname != '':
            self.model.IntervalLoad(fname=fname)

    def add_badchannel(self):
        """Opens dialog for user input of channels to mark as bad."""
        text = 'Channel number: \n(e.g.: 3, 5, 8-12)'
        uinp, ok = QInputDialog.getText(None, 'Add as bad channel', text)
        if ok:
            uinp = uinp.replace(' ', '')  # removes blank spaces
            ch_str = uinp.split(',')     # splits csv
            try:
                ch_list = []
                for elem in ch_str:
                    if '-' in elem:   # if given a range e.g. 7-12
                        elem_lims = elem.split('-')
                        seq = range(int(elem_lims[0]), int(elem_lims[1]) + 1)
                        ch_list.extend(seq)
                    else:             # if given a single value
                        ch_list.append(int(elem))
                self.model.BadChannelAdd(ch_list=ch_list)
            except Exception as ex:
                print(str(ex))

    def del_badchannel(self):
        """Opens dialog for user input of channels to unmark as bad."""
        text = 'Channel number: \n(e.g.: 3, 5, 8-12)'
        uinp, ok = QInputDialog.getText(None, 'Delete bad channel', text)
        if ok:
            uinp = uinp.replace(' ', '')  # removes blank spaces
            ch_str = uinp.split(',')     # splits csv
            try:
                ch_list = []
                for elem in ch_str:
                    if '-' in elem:   # if given a range e.g. 7-12
                        elem_lims = elem.split('-')
                        seq = range(int(elem_lims[0]), int(elem_lims[1]) + 1)
                        ch_list.extend(seq)
                    else:             # if given a single value
                        ch_list.append(int(elem))
                self.model.BadChannelDel(ch_list=ch_list)
            except Exception as ex:
                print(str(ex))

    def spectral_decomposition(self):
        """Opens Spectral decomposition dialog."""
        w = SpectralChoiceDialog(self)
        if w.value == 1:       # If new data was created
            self.model.refresh_file()

    def event_related_potential(self):
        """Opens ERP window."""
        open_dialog = True
        # if no intervals exist, raise a no intervals data message
        if len(self.model.nwb.intervals)==0:
            NoTrialsDialog()
            open_dialog = False
        try:  # Tries to load High Gamma and intervals data
            hg = self.model.nwb.processing['ecephys'].data_interfaces['high_gamma']
        except:  # if it fails, raises a no high gamma message
            NoHighGammaDialog()
            open_dialog = False
        # If all tries passed, open ERP
        if open_dialog:
            w = ERPDialog(parent=self)

    def audio_event_detection(self):
        """Opens Audio Event Detection window."""
        # Test if trials already exist
        if 'TimeIntervals_speaker' not in self.model.nwb.intervals:
            # Test if file contains audio signals
            if any(name in self.model.nwb.stimulus for name in ['speaker1', 'speaker2']):
                AudioEventDetection(parent=self)
            else:
                NoAudioDialog()
        else:
            ExistIntervalsDialog()

    def add_survey(self):
        """Add survey data from .mat file to current nwb file."""
        # Open file dialog
        path_file, _ = QFileDialog.getOpenFileName(None, 'Open file', '', "(*.mat)")
        if os.path.isfile(path_file):
            add_survey_data(nwbfile=self.model.nwb, path_survey_file=path_file)
            self.action_vis_survey.setEnabled(True)
            self.action_add_survey.setEnabled(False)
            # Write changes to NWB file
            self.model.io.write(self.model.nwb)

    def visualize_survey(self):
        """Visualize survey data in current nwb file."""
        # Test if current nwb file contains Survey table
        if 'behavior' in self.model.nwb.processing:
            list_surveys = [v for v in self.model.nwb.processing['behavior'].data_interfaces.values()
                            if v.neurodata_type == 'SurveyTable']
            if len(list_surveys) > 0:
                ShowSurveyDialog(nwbfile=self.model.nwb)

    def add_transcription_timitsounds(self):
        """Add TimitSounds transcription data to current nwb file."""
        dir_path = QFileDialog.getExistingDirectory(self, 'Open TimitSounds dir', '', QtGui.QFileDialog.ShowDirsOnly)
        if os.path.isdir(dir_path):
            _ = add_transcription_data(
                nwbfile=self.model.nwb,
                path_transcription=dir_path,
                tr_type='timitsounds'
            )
            self.action_vis_transcription.setEnabled(True)
            self.transcriptionadd_tools_menu.setEnabled(False)
            # Write changes to NWB file
            self.model.io.write(self.model.nwb)
            print('Transcription data added successfully!')

    def add_transcription_mocha(self):
        """Add Mocha transcription data to current nwb file."""
        dir_path = QFileDialog.getExistingDirectory(self, 'Open Mocha dir', '', QtGui.QFileDialog.ShowDirsOnly)
        if os.path.isdir(dir_path):
            nwbfile = add_transcription_data(
                nwbfile=self.model.nwb,
                path_transcription=dir_path,
                tr_type='mocha',
                subject_id=self.model.subject_id,  # 'EC118',
                session_id='B' + self.model.block,  # 'B6'
            )
            if nwbfile is None:
                print('No transcription data for {}_B{}'.format(self.model.subject_id, self.model.block))
            else:
                self.action_vis_transcription.setEnabled(True)
                self.transcriptionadd_tools_menu.setEnabled(False)
                # Write changes to NWB file
                self.model.nwb = nwbfile
                self.model.io.write(self.model.nwb)
                print('Transcription data added successfully!')

    def add_transcription_textgrid(self):
        """Add TextGrid transcription data to current nwb file."""
        filename, _ = QFileDialog.getOpenFileName(None, 'Open file', '', "(*.TextGrid)")
        if os.path.isfile(filename):
            add_transcription_data(
                nwbfile=self.model.nwb,
                path_transcription=filename,
                tr_type='textgrid'
            )
            self.action_vis_transcription.setEnabled(True)
            self.transcriptionadd_tools_menu.setEnabled(False)
            # Write changes to NWB file
            self.model.io.write(self.model.nwb)
            print('Transcription data added successfully!')

    def visualize_transcription(self):
        """Visualize transcription behavioral data in current nwb file."""
        # # Test if current nwb file contains Survey table
        # if 'behavior' in self.model.nwb.processing:
        #     list_surveys = [v for v in self.model.nwb.processing['behavior'].data_interfaces.values()
        #                     if v.neurodata_type == 'SurveyTable']
        #     if len(list_surveys) > 0:
        #         ShowSurveyDialog(nwbfile=self.model.nwb)
        ShowTranscriptionDialog(nwbfile=self.model.nwb)
        pass

    def visualize_electrodes(self):
        """Visualize electrodes tables in current nwb file."""
        ShowElectrodesDialog(parent=self)

    def about(self):
        """About dialog."""
        msg = QMessageBox()
        msg.setWindowTitle("About ecogVIS")
        msg.setIcon(QMessageBox.Information)
        msg.setText("Version: 1.0.0 \n"
                    "Timeseries visualizer for ECoG signals stored in NWB files.\n ")
        msg.setInformativeText("<a href='https://github.com/luiztauffer/ecogVIS/'>ecogVIS Github page</a>")
        msg.setStandardButtons(QMessageBox.Ok)
        msg.exec_()

    def reset_buttons(self):
        """Controls appearance and active state of panel buttons."""
        global intervalAdd_
        global intervalDel_
        global annotationAdd_
        global annotationDel_

        # self.active_mode = 'default', 'intervalAdd', 'intervalDel'
        if self.active_mode != 'intervalAdd':
            self.push2_1.setChecked(False)
            intervalAdd_ = False

        if self.active_mode != 'intervalDel':
            self.push2_2.setChecked(False)
            intervalDel_ = False

        if self.active_mode != 'annotationAdd':
            self.push1_1.setChecked(False)
            annotationAdd_ = False

        if self.active_mode != 'annotationDel':
            self.push1_2.setChecked(False)
            annotationDel_ = False

    # Annotation functions ----------------------------------------------------
    def AnnotationColor(self):
        """Selects annotation color."""
        global annotationColor_
        annotationColor_ = str(self.combo1.currentText())

    def AnnotationAdd(self):
        """Add new annotation at point selected with mouse right-click."""
        global annotationAdd_
        if self.push1_1.isChecked():  # if button is pressed down
            self.active_mode = 'annotationAdd'
            self.reset_buttons()
            annotationAdd_ = True
        else:
            self.active_mode = 'default'
            self.reset_buttons()

    def AnnotationDel(self):
        """Deletes annotations chosen with mouse right-click."""
        global annotationDel_
        if self.push1_2.isChecked():  # if button is pressed down
            self.active_mode = 'annotationDel'
            self.reset_buttons()
            annotationDel_ = True
        else:
            self.active_mode = 'default'
            self.reset_buttons()

    def AnnotationSave(self):
        """Saves current set of annotations to file."""
        self.active_mode = 'default'
        self.reset_buttons()
        try:
            self.model.AnnotationSave()
        except Exception as ex:
            self.log_error(str(ex))

    # Interval functions ------------------------------------------------------
    def IntervalType(self):
        """Add new interval type."""
        global intervalType_
        global intervalsDict_

        item = str(self.combo2.currentText())
        if item == 'add custom':
            w = CustomIntervalDialog()
            int_type, color = w.getResults()
            if len(int_type) > 0:    # If user chose a valid name for the new interval type
                curr_ind = self.combo2.currentIndex()
                self.combo2.setItemText(curr_ind, int_type)
                self.combo2.addItem('add custom')
                intervalType_ = int_type
                intervalsDict_[intervalType_] = {'type': intervalType_,
                                                 'session': self.current_session,
                                                 'color': color,
                                                 'counts': 0}
                self.combo2.setCurrentIndex(curr_ind)
            else:
                self.combo2.setCurrentIndex(0)
        else:
            intervalType_ = item

    def IntervalAdd(self):
        """Adds intervals with mouse click-and-drag at time series plot."""
        global intervalAdd_
        if self.push2_1.isChecked():  # if button is pressed down
            self.active_mode = 'intervalAdd'
            self.reset_buttons()
            intervalAdd_ = True
        else:
            self.active_mode = 'default'
            self.reset_buttons()

    def IntervalDel(self):
        """Deletes intervals chosen with mouse right-click."""
        global intervalDel_
        if self.push2_2.isChecked():  # if button is pressed down
            self.active_mode = 'intervalDel'
            self.reset_buttons()
            intervalDel_ = True
        else:
            self.active_mode = 'default'
            self.reset_buttons()

    def IntervalSave(self):
        """Saves current set of intervals to file."""
        self.active_mode = 'default'
        self.reset_buttons()
        try:
            self.model.IntervalSave()
        except Exception as ex:
            self.log_error(str(ex))

    def ChannelSelect(self):
        """Opens dialog to select channels from specific brain regions."""
        self.active_mode = 'default'
        self.reset_buttons()
        # Dialog to choose channels from specific brain regions
        w = SelectChannelsDialog(
            stringlist=self.model.all_regions,
            checked=self.model.regions_mask
        )
        all_locs = self.model.electrodes_table['location'][self.model.electrical_series_channel_ids]
        self.model.channels_mask = np.zeros(len(all_locs))
        for loc in w.choices:
            self.model.channels_mask += all_locs == np.array(loc)
        # Indices of channels from chosen regions
        self.model.channels_mask_ind = np.where(self.model.channels_mask)[0]
        self.model.n_channels_total = len(self.model.channels_mask_ind)
        # Reset channels span control
        self.model.lastCh = np.minimum(16, self.model.n_channels_total)
        self.model.firstCh = 1
        self.model.nChToShow = self.model.lastCh - self.model.firstCh + 1
        self.qline0.setText(str(self.model.lastCh))
        self.qline1.setText(str(self.model.firstCh))
        # Update signals plot
        self.model.selectedChannels = self.model.channels_mask_ind[self.model.firstCh - 1:self.model.lastCh]
        self.model.refreshScreen()

    def PeriodogramSelect(self):
        """Opens Periodogram grid window. Checks if PSD data is present on file
        and, if not, asks the user if it should be calculated."""
        if self.combo3.currentText() == 'raw':
            try:  # tries to read fft and welch spectral data
                psd_fft = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_fft_raw']
                psd_welch = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_welch_raw']
                PeriodogramGridDialog(self)
            except:
                w = NoSpectrumDialog(self, 'raw')
                if w.val == 1:  # PSD was calculated
                    self.model.refresh_file()  # re-opens the file, now with new data
                    self.combo3.setCurrentIndex(self.combo3.findText('raw'))
                    psd_fft = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_fft_raw']
                    psd_welch = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_welch_raw']
                    PeriodogramGridDialog(self)
        elif self.combo3.currentText() == 'preprocessed':
            try:  # tries to read fft and welch spectral data
                psd_fft = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_fft_preprocessed']
                psd_welch = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_welch_preprocessed']
                PeriodogramGridDialog(self)
            except:
                w = NoSpectrumDialog(self, 'preprocessed')
                if w.val == 1:  # PSD was calculated
                    self.model.refresh_file()  # re-opens the file, now with new data
                    self.combo3.setCurrentIndex(self.combo3.findText('preprocessed'))
                    psd_fft = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_fft_preprocessed']
                    psd_welch = self.model.nwb.modules['ecephys'].data_interfaces['Spectrum_welch_preprocessed']
                    PeriodogramGridDialog(self)

    def Preprocess(self):
        """Opens Preprocessing dialog."""
        w = PreprocessingDialog(self)
        if w.value == 1:       # If new data was created
            self.model.refresh_file()        # re-opens the file, now with new data
            self.combo3.setCurrentIndex(self.combo3.findText('preprocessed'))
            self.voltage_time_series()

    def CalcHighGamma(self):
        """Opens calculate High Gamma dialog."""
        w = HighGammaDialog(self)
        if w.value == 1:       # If new data was created
            self.open_another_file(filename=w.new_fname)

    def voltage_time_series(self):
        """
        Selects the time series data to be ploted on main window. The options are:
        - 'voltage_raw': Raw voltage traces, stored in nwb.acquisition['ElectricalSeries']
        - 'preprocessed': Preprocessed voltage traces, stored in nwb.processing['ecephys'].data_interfaces['LFP'].electrical_series['preprocessed']
        - 'high gamma': High Gamma estimation traces, stored in nwb.processing['ecephys'].data_interfaces['high_gamma']
        """
        if self.combo3.currentText() == 'raw':
            lis = list(self.model.nwb.acquisition.keys())
            there_is_raw = False
            for i in lis:  # Check if there is ElectricalSeries in acquisition group
                if type(self.model.nwb.acquisition[i]).__name__ == 'ElectricalSeries':
                    self.model.source = self.model.nwb.acquisition[i]
                    there_is_raw = True
            if there_is_raw:
                self.model.plot_panel = 'voltage_raw'
                self.model.plotData = self.model.source.data
                self.model.nBins = self.model.plotData.shape[0]     # total number of bins
                self.model.fs_signal = self.model.source.rate     # sampling frequency [Hz]
                self.model.tbin_signal = 1 / self.model.fs_signal  # time bin duration [seconds]
                self.push5_0.setEnabled(True)
                self.push6_0.setEnabled(True)
                self.push7_0.setEnabled(False)
            else:
                self.combo3.setCurrentIndex(self.combo3.findText('high gamma'))
                self.voltage_time_series()
                NoRawDialog()
        elif self.combo3.currentText() == 'preprocessed':
            try:   # if preprocessed signals already exist on NWB file
                self.model.source = self.model.nwb.processing['ecephys'].data_interfaces['LFP'].electrical_series['preprocessed']
                self.model.plotData = self.model.source.data
                self.model.plot_panel = 'voltage_preprocessed'
                self.model.nBins = self.model.source.data.shape[0]
                self.model.fs_signal = self.model.source.rate
                self.model.tbin_signal = 1 / self.model.fs_signal
                self.push5_0.setEnabled(True)
                self.push6_0.setEnabled(False)
                self.push7_0.setEnabled(True)
            except:
                self.combo3.setCurrentIndex(self.combo3.findText('raw'))
                self.voltage_time_series()
                NoPreprocessedDialog()
        elif self.combo3.currentText() == 'high gamma':
            try:     # if high gamma already exists on NWB file
                self.model.source = self.model.nwb.processing['ecephys'].data_interfaces['high_gamma']
                self.model.plotData = self.model.source.data
                self.model.plot_panel = 'spectral_power'
                self.model.nBins = self.model.source.data.shape[0]
                self.model.fs_signal = self.model.source.rate
                self.model.tbin_signal = 1 / self.model.fs_signal
                self.push5_0.setEnabled(False)
                self.push6_0.setEnabled(False)
                self.push7_0.setEnabled(False)
            except:  # if not, opens warning dialog
                self.combo3.setCurrentIndex(self.combo3.findText('preprocessed'))
                self.voltage_time_series()
                NoHighGammaDialog()
        self.model.updateCurXAxisPosition()    # updates time points
        self.model.refreshScreen()

    def choose_stim(self):
        """Choose stimulus."""
        stimName = self.combo4.currentText()
        if stimName != '':
            self.model.refreshScreen()

    # Plot control buttons ----------------------------------------------------
    def scroll_up(self):
        self.model.channel_Scroll_Up('unit')

    def scroll_up_page(self):
        self.model.channel_Scroll_Up('page')

    def scroll_down(self):
        self.model.channel_Scroll_Down('unit')

    def scroll_down_page(self):
        self.model.channel_Scroll_Down('page')

    def page_backward(self):
        self.model.time_scroll(scroll=-1)

    def scroll_backward(self):
        self.model.time_scroll(scroll=-1 / 3)

    def page_forward(self):
        self.model.time_scroll(scroll=1)

    def scroll_forward(self):
        self.model.time_scroll(scroll=1 / 3)

    def horizontalScaleIncrease(self):
        self.model.horizontalScaleIncrease()

    def horizontalScaleDecrease(self):
        self.model.horizontalScaleDecrease()

    def verticalScale(self):
        """Updates vertical scale and refreshes plot."""
        self.model.refreshScreen()

    def verticalScaleIncrease(self):
        """Increases vertical scale by a factor of 2."""
        scaleFac = float(self.qline4.text())
        self.qline4.setText(str(scaleFac * 2))
        self.model.refreshScreen()

    def verticalScaleDecrease(self):
        """Decreases vertical scale by a factor of 2."""
        scaleFac = float(self.qline4.text())
        self.qline4.setText(str(scaleFac / 2))
        self.model.refreshScreen()

    def time_window_size(self):
        self.model.updateCurXAxisPosition()

    def time_window_enlarge(self):
        self.model.time_window_resize(2.)

    def time_window_reduce(self):
        self.model.time_window_resize(0.5)

    def interval_start(self):
        self.model.updateCurXAxisPosition()

    def channelDisplayed(self):
        self.model.nChannels_Displayed()


class CustomViewBox(pg.ViewBox):
    """Customized ViewBox to add/del Annotations and Intervals."""
    def __init__(self, parent):
        pg.ViewBox.__init__(self)
        self.parent = parent

    def mouseClickEvent(self, ev):
        global intervalDel_
        global annotationAdd_
        global annotationDel_
        global annotationColor_

        if intervalDel_:
            mousePoint = self.mapSceneToView(ev.scenePos())
            x = mousePoint.x()
            try:
                self.parent.model.IntervalDel(round(x, 3))
            except Exception as ex:
                print(str(ex))

        if annotationAdd_:
            mousePoint = self.mapSceneToView(ev.scenePos())
            x = mousePoint.x()
            y = mousePoint.y()
            try:
                text, ok = QInputDialog.getText(None, 'Annotations', 'Enter your annotation:')
                self.parent.model.AnnotationAdd(x=x, y=y, color=annotationColor_, text=text)
            except Exception as ex:
                print(str(ex))

        if annotationDel_:
            mousePoint = self.mapSceneToView(ev.scenePos())
            x = mousePoint.x()
            y = mousePoint.y()
            try:
                self.parent.model.AnnotationDel(x=x, y=y)
            except Exception as ex:
                print(str(ex))

    def mouseDragEvent(self, ev):
        global intervalType_
        global intervalAdd_
        global intervalsDict_

        if intervalAdd_:
            if ev.button() == QtCore.Qt.RightButton:
                ev.ignore()
            else:
                pg.ViewBox.mouseDragEvent(self, ev)
                if ev.isStart():    # first click before dragging
                    a = self.mapSceneToView(ev.scenePos())
                    self.pos1 = a.x()              # initial x mark
                    self.parent.model.DrawMarkTime(self.pos1)     # temporary line on x mark
                elif ev.isFinish():  # release from dragging
                    self.parent.model.RemoveMarkTime()   # remove line on x mark
                    a = self.mapSceneToView(ev.scenePos())
                    self.pos2 = a.x()              # final x mark
                    if self.pos1 is not None:
                        if self.pos2 < self.pos1:     # marking from right to left
                            interval = [round(self.pos2, 3), round(self.pos1, 3)]
                        else:               # marking from left to right
                            interval = [round(self.pos1, 3), round(self.pos2, 3)]
                        color = intervalsDict_[intervalType_]['color']
                        session = self.parent.current_session
                        intervalsDict_[intervalType_]['counts'] += 1
                        self.parent.model.IntervalAdd(interval, intervalType_, color, session)
                        self.parent.model.refreshScreen()


# If it is imported as a module
def main(source_path='', metafile=None):
    import sys

    # Sets up QT application
    app = QCoreApplication.instance()
    if app is None:
        app = QApplication(sys.argv)  # instantiate a QtGui (holder for the app)
    ex = Application(source_path=source_path, metafile=metafile)
    sys.exit(app.exec_())


def parse_arguments():
    import argparse

    parser = argparse.ArgumentParser(
        description='Timeseries visualizer and data processing tools for Electrocorticography '
                    '(ECoG) signals stored in NWB files, for Python.',
    )

    parser.add_argument(
        "--source",
        default='',
        help="NWB file or HTK dir to be read."
    )
    parser.add_argument(
        "--metafile",
        default=None,
        help="The path to the metadata YAML file."
    )

    # Parse arguments
    args = parser.parse_args()

    # File or dir to be loaded
    source_path = args.source

    # Metadata file (.yml)
    metafile = args.metafile

    return source_path, metafile


def cmd_line_shortcut():
    source_path, metafile = parse_arguments()
    main(source_path=source_path, metafile=metafile)


if __name__ == '__main__':
    source_path, metafile = parse_arguments()
    main(source_path=source_path, metafile=metafile)



from __future__ import division
from process_nwb.wavelet_transform import gaussian,hamming
import numpy as np

from process_nwb.fft import fftfreq, fft, ifft
try:
    from accelerate.mkl.fftpack import fft, ifft
except ImportError:
    try:
        from pyfftw.interfaces.numpy_fft import fft, ifft
    except ImportError:
        from numpy.fft import fft, ifft


__authors__ = "Alex Bujan, Jesse Livezey"
__all__ = ['hilbert_transform']


def hilbert_transform(X, rate, filters=None, phase=None, X_fft_h=None):
    """
    Apply bandpass filtering with Hilbert transform using
    a prespecified set of filters.

    Parameters
    ----------
    X : ndarray (n_channels, n_time)
        Input data, dimensions
    rate : float
        Number of samples per second.
    filters : filter or list of filters (optional)
        One or more bandpass filters

    Returns
    -------
    Xh : ndarray, complex
        Bandpassed analytic signal
    """
    if not isinstance(filters, list):
        filters = [filters]
    time = X.shape[-1]
    freq = fftfreq(time, 1. / rate)

    Xh = np.zeros((len(filters),) + X.shape, dtype=np.complex)
    if X_fft_h is None:
        # Heavyside filter
        h = np.zeros(len(freq))
        h[freq > 0] = 2.
        h[0] = 1.
        h = h[np.newaxis, :]
        X_fft_h = fft(X) * h
        if phase is not None:
            X_fft_h *= phase
    for ii, f in enumerate(filters):
        if f is None:
            Xh[ii] = ifft(X_fft_h)
        else:
            f = f / np.linalg.norm(f)
            Xh[ii] = ifft(X_fft_h * f)
    if Xh.shape[0] == 1:
        return Xh[0], X_fft_h

    return Xh, X_fft_h



# -*- coding: utf-8 -*-
import numpy as np
from hdmf.common.table import DynamicTable, VectorData
from ndx_spectrum import Spectrum
from ndx_survey_data.survey_definitions import nrs_survey_table, mpq_survey_table, vas_survey_table
from ndx_bipolar_scheme import BipolarSchemeTable, EcephysExt
from ndx_ecog import CorticalSurfaces, ECoGSubject
from pynwb import NWBFile, NWBHDF5IO, get_manager, ProcessingModule
from pynwb.base import TimeSeries
from pynwb.device import Device
from pynwb.ecephys import LFP, ElectricalSeries
from pynwb.epoch import TimeIntervals
from pynwb.file import Subject, DynamicTableRegion
from pynwb.misc import DecompositionSeries
import string
import random


def id_generator(size=6, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for _ in range(size))


def nwb_copy_file(old_file, new_file, cp_objs={}, save_to_file=True):
    """
    Copy fields defined in 'obj', from existing NWB file to new NWB file.

    Parameters
    ----------
    old_file : str, path, nwbfile
        String or path to nwb file '/path/to/old_file.nwb'. Alternatively, the
        nwbfile object.
    new_file : str, path
        String such as '/path/to/new_file.nwb'.
    cp_objs : dict
        Name:Value pairs (Group:Children) listing the groups and respective
        children from the current NWB file to be copied. Children can be:
        - Boolean, indicating an attribute (e.g. for institution, lab)
        - List of strings, containing several children names
        Example:
        {'institution':True,
         'lab':True,
         'acquisition':['microphone'],
         'ecephys':['LFP','DecompositionSeries']}
    save_to_file: Boolean
        If True, saves directly to new_file.nwb. If False, only returns nwb_new.

    Returns:
    --------
    nwb_new : nwbfile object
    """

    manager = get_manager()

    # Get from nwbfile object in memory or from file
    if isinstance(old_file, NWBFile):
        nwb_old = old_file
        io1 = False
    else:
        io1 = NWBHDF5IO(str(old_file), 'r', manager=manager, load_namespaces=True)
        nwb_old = io1.read()

    # Creates new file
    nwb_new = NWBFile(
        session_description=str(nwb_old.session_description),
        identifier=id_generator(),
        session_start_time=nwb_old.session_start_time,
    )
    with NWBHDF5IO(new_file, mode='w', manager=manager, load_namespaces=False) as io2:
        # Institution name ------------------------------------------------
        if 'institution' in cp_objs:
            nwb_new.institution = str(nwb_old.institution)

        # Lab name --------------------------------------------------------
        if 'lab' in cp_objs:
            nwb_new.lab = str(nwb_old.lab)

        # Session id ------------------------------------------------------
        if 'session' in cp_objs:
            nwb_new.session_id = nwb_old.session_id

        # Devices ---------------------------------------------------------
        if 'devices' in cp_objs:
            for aux in list(nwb_old.devices.keys()):
                dev = Device(nwb_old.devices[aux].name)
                nwb_new.add_device(dev)

        # Electrode groups ------------------------------------------------
        if 'electrode_groups' in cp_objs and nwb_old.electrode_groups is not None:
            for aux in list(nwb_old.electrode_groups.keys()):
                nwb_new.create_electrode_group(
                    name=str(nwb_old.electrode_groups[aux].name),
                    description=str(nwb_old.electrode_groups[
                        aux].description),
                    location=str(nwb_old.electrode_groups[aux].location),
                    device=nwb_new.get_device(
                        nwb_old.electrode_groups[aux].device.name)
                )

        # Electrodes ------------------------------------------------------
        if 'electrodes' in cp_objs and nwb_old.electrodes is not None:
            nElec = len(nwb_old.electrodes['x'].data[:])
            for aux in np.arange(nElec):
                nwb_new.add_electrode(
                    x=nwb_old.electrodes['x'][aux],
                    y=nwb_old.electrodes['y'][aux],
                    z=nwb_old.electrodes['z'][aux],
                    imp=nwb_old.electrodes['imp'][aux],
                    location=str(nwb_old.electrodes['location'][aux]),
                    filtering=str(nwb_old.electrodes['filtering'][aux]),
                    group=nwb_new.get_electrode_group(
                        nwb_old.electrodes['group'][aux].name),
                    group_name=str(nwb_old.electrodes['group_name'][aux])
                )
            # if there are custom variables
            new_vars = list(nwb_old.electrodes.colnames)
            default_vars = ['x', 'y', 'z', 'imp', 'location', 'filtering',
                            'group', 'group_name']
            [new_vars.remove(var) for var in default_vars]
            for var in new_vars:
                if var == 'label':
                    var_data = [str(elem) for elem in nwb_old.electrodes[var].data[:]]
                else:
                    var_data = np.array(nwb_old.electrodes[var].data[:])

                nwb_new.add_electrode_column(
                    name=str(var),
                    description=str(nwb_old.electrodes[var].description),
                    data=var_data
                )

            # If Bipolar scheme for electrodes
            for v in nwb_old.lab_meta_data.values():
                if isinstance(v, EcephysExt) and hasattr(v, 'bipolar_scheme_table'):
                    bst_old = v.bipolar_scheme_table
                    bst_new = BipolarSchemeTable(
                        name=bst_old.name,
                        description=bst_old.description
                    )
                    ecephys_ext = EcephysExt(name=v.name)
                    ecephys_ext.bipolar_scheme_table = bst_new
                    nwb_new.add_lab_meta_data(ecephys_ext)

        # Epochs ----------------------------------------------------------
        if 'epochs' in cp_objs and nwb_old.epochs is not None:
            nEpochs = len(nwb_old.epochs['start_time'].data[:])
            for i in np.arange(nEpochs):
                nwb_new.add_epoch(
                    start_time=nwb_old.epochs['start_time'].data[i],
                    stop_time=nwb_old.epochs['stop_time'].data[i])
            # if there are custom variables
            new_vars = list(nwb_old.epochs.colnames)
            default_vars = ['start_time', 'stop_time', 'tags',
                            'timeseries']
            [new_vars.remove(var) for var in default_vars if
             var in new_vars]
            for var in new_vars:
                nwb_new.add_epoch_column(
                    name=var,
                    description=nwb_old.epochs[var].description,
                    data=nwb_old.epochs[var].data[:]
                )

        # Invalid times ---------------------------------------------------
        if 'invalid_times' in cp_objs and nwb_old.invalid_times is not None:
            nInvalid = len(nwb_old.invalid_times['start_time'][:])
            for aux in np.arange(nInvalid):
                nwb_new.add_invalid_time_interval(
                    start_time=nwb_old.invalid_times['start_time'][aux],
                    stop_time=nwb_old.invalid_times['stop_time'][aux]
                )

        # Trials ----------------------------------------------------------
        if 'trials' in cp_objs and nwb_old.trials is not None:
            nTrials = len(nwb_old.trials['start_time'])
            for aux in np.arange(nTrials):
                nwb_new.add_trial(
                    start_time=nwb_old.trials['start_time'][aux],
                    stop_time=nwb_old.trials['stop_time'][aux])
            # if there are custom variables
            new_vars = list(nwb_old.trials.colnames)
            default_vars = ['start_time', 'stop_time']
            [new_vars.remove(var) for var in default_vars]
            for var in new_vars:
                nwb_new.add_trial_column(
                    name=var,
                    description=nwb_old.trials[var].description,
                    data=nwb_old.trials[var].data[:]
                )

        # Intervals -------------------------------------------------------
        if 'intervals' in cp_objs and nwb_old.intervals is not None:
            all_objs_names = list(nwb_old.intervals.keys())
            for obj_name in all_objs_names:
                obj_old = nwb_old.intervals[obj_name]
                # create and add TimeIntervals
                obj = TimeIntervals(name=obj_old.name,
                                    description=obj_old.description)
                nInt = len(obj_old['start_time'])
                for ind in np.arange(nInt):
                    obj.add_interval(start_time=obj_old['start_time'][ind],
                                     stop_time=obj_old['stop_time'][ind])
                # Add to file
                nwb_new.add_time_intervals(obj)

        # Stimulus --------------------------------------------------------
        if 'stimulus' in cp_objs:
            all_objs_names = list(nwb_old.stimulus.keys())
            for obj_name in all_objs_names:
                obj_old = nwb_old.stimulus[obj_name]
                obj = TimeSeries(name=obj_old.name,
                                 description=obj_old.description,
                                 data=obj_old.data[:],
                                 rate=obj_old.rate,
                                 resolution=obj_old.resolution,
                                 conversion=obj_old.conversion,
                                 starting_time=obj_old.starting_time,
                                 unit=obj_old.unit)
                nwb_new.add_stimulus(obj)

        # Processing modules ----------------------------------------------
        if 'ecephys' in cp_objs:
            interfaces = [nwb_old.processing['ecephys'].data_interfaces[key]
                          for key in cp_objs['ecephys']]
            # Add ecephys module to NWB file
            ecephys_module = ProcessingModule(
                name='ecephys',
                description='Extracellular electrophysiology data.'
            )
            nwb_new.add_processing_module(ecephys_module)
            for interface_old in interfaces:
                obj = copy_obj(interface_old, nwb_old, nwb_new)
                if obj is not None:
                    ecephys_module.add_data_interface(obj)

        if 'behavior' in cp_objs:
            interfaces = [nwb_old.processing['behavior'].data_interfaces[key]
                          for key in cp_objs['behavior']]
            if 'behavior' not in nwb_new.processing:
                # Add behavior module to NWB file
                behavior_module = ProcessingModule(
                    name='behavior',
                    description='behavioral data.'
                )
                nwb_new.add_processing_module(behavior_module)
            for interface_old in interfaces:
                obj = copy_obj(interface_old, nwb_old, nwb_new)
                if obj is not None:
                    behavior_module.add_data_interface(obj)

        # Acquisition -----------------------------------------------------
        # Can get raw ElecetricalSeries and Mic recording
        if 'acquisition' in cp_objs:
            for acq_name in cp_objs['acquisition']:
                obj_old = nwb_old.acquisition[acq_name]
                acq = copy_obj(obj_old, nwb_old, nwb_new)
                nwb_new.add_acquisition(acq)

        # Surveys ---------------------------------------------------------
        if 'surveys' in cp_objs and 'behavior' in nwb_old.processing:
            surveys_list = [v for v in nwb_old.processing['behavior'].data_interfaces.values()
                            if v.neurodata_type == 'SurveyTable']
            if cp_objs['surveys'] and len(surveys_list) > 0:
                if 'behavior' not in nwb_new.processing:
                    # Add behavior module to NWB file
                    behavior_module = ProcessingModule(
                        name='behavior',
                        description='behavioral data.'
                    )
                    nwb_new.add_processing_module(behavior_module)
                for obj_old in surveys_list:
                    srv = copy_obj(obj_old, nwb_old, nwb_new)
                    behavior_module.add_data_interface(srv)

        # Subject ---------------------------------------------------------
        if nwb_old.subject is not None:
            if 'subject' in cp_objs:
                try:
                    cortical_surfaces = CorticalSurfaces()
                    surfaces = nwb_old.subject.cortical_surfaces.surfaces
                    for sfc in list(surfaces.keys()):
                        cortical_surfaces.create_surface(
                            name=surfaces[sfc].name,
                            faces=surfaces[sfc].faces,
                            vertices=surfaces[sfc].vertices)
                    nwb_new.subject = ECoGSubject(
                        cortical_surfaces=cortical_surfaces,
                        subject_id=nwb_old.subject.subject_id,
                        age=nwb_old.subject.age,
                        description=nwb_old.subject.description,
                        genotype=nwb_old.subject.genotype,
                        sex=nwb_old.subject.sex,
                        species=nwb_old.subject.species,
                        weight=nwb_old.subject.weight,
                        date_of_birth=nwb_old.subject.date_of_birth)
                except:
                    nwb_new.subject = Subject(**nwb_old.subject.fields)

        # Write new file with copied fields
        if save_to_file:
            io2.write(nwb_new, link_data=False)

    # Close old file and return new nwbfile object
    if io1:
        io1.close()

    return nwb_new


def copy_obj(obj_old, nwb_old, nwb_new):
    """ Creates a copy of obj_old. """

    # ElectricalSeries --------------------------------------------------------
    if type(obj_old) is ElectricalSeries:
        # If reference electrodes table is bipolar scheme
        if isinstance(obj_old.electrodes.table, BipolarSchemeTable):
            bst_old = obj_old.electrodes.table
            bst_old_df = bst_old.to_dataframe()
            bst_new = nwb_new.lab_meta_data['ecephys_ext'].bipolar_scheme_table

            for id, row in bst_old_df.iterrows():
                index_anodes = row['anodes'].index.tolist()
                index_cathodes = row['cathodes'].index.tolist()
                bst_new.add_row(anodes=index_anodes, cathodes=index_cathodes)
            bst_new.anodes.table = nwb_new.electrodes
            bst_new.cathodes.table = nwb_new.electrodes

            # if there are custom columns
            new_cols = list(bst_old_df.columns)
            default_cols = ['anodes', 'cathodes']
            [new_cols.remove(col) for col in default_cols]
            for col in new_cols:
                col_data = list(bst_old[col].data[:])
                bst_new.add_column(
                    name=str(col),
                    description=str(bst_old[col].description),
                    data=col_data
                )

            elecs_region = DynamicTableRegion(
                name='electrodes',
                data=bst_old_df.index.tolist(),
                description='desc',
                table=bst_new
            )
        else:
            region = np.array(obj_old.electrodes.table.id[:])[obj_old.electrodes.data[:]].tolist()
            elecs_region = nwb_new.create_electrode_table_region(
                name='electrodes',
                region=region,
                description=''
            )
        els = ElectricalSeries(
            name=obj_old.name,
            data=obj_old.data[:],
            electrodes=elecs_region,
            rate=obj_old.rate,
            description=obj_old.description
        )
        return els

    # DynamicTable ------------------------------------------------------------
    if type(obj_old) is DynamicTable:
        return DynamicTable(
            name=obj_old.name,
            description=obj_old.description,
            colnames=obj_old.colnames,
            columns=obj_old.columns,
        )

    # LFP ---------------------------------------------------------------------
    if type(obj_old) is LFP:
        obj = LFP(name=obj_old.name)
        assert len(obj_old.electrical_series) == 1, (
            'Expected precisely one electrical series, got %i!' %
            len(obj_old.electrical_series))
        els = list(obj_old.electrical_series.values())[0]

        ####
        # first check for a table among the new file's data_interfaces
        if els.electrodes.table.name in nwb_new.processing['ecephys'].data_interfaces:
            LFP_dynamic_table = nwb_new.processing['ecephys'].data_interfaces[els.electrodes.table.name]
        else:
            # othewise use the electrodes as the table
            LFP_dynamic_table = nwb_new.electrodes
        ####

        region = np.array(els.electrodes.table.id[:])[els.electrodes.data[:]].tolist()
        elecs_region = LFP_dynamic_table.create_region(
            name='electrodes',
            region=region,
            description=els.electrodes.description
        )

        obj.create_electrical_series(
            name=els.name,
            comments=els.comments,
            conversion=els.conversion,
            data=els.data[:],
            description=els.description,
            electrodes=elecs_region,
            rate=els.rate,
            resolution=els.resolution,
            starting_time=els.starting_time
        )

        return obj

    # TimeSeries --------------------------------------------------------------
    if type(obj_old) is TimeSeries:
        return TimeSeries(
            name=obj_old.name,
            description=obj_old.description,
            data=obj_old.data[:],
            rate=obj_old.rate,
            resolution=obj_old.resolution,
            conversion=obj_old.conversion,
            starting_time=obj_old.starting_time,
            unit=obj_old.unit
        )

    # DecompositionSeries -----------------------------------------------------
    if type(obj_old) is DecompositionSeries:
        list_columns = []
        for item in obj_old.bands.columns:
            bp = VectorData(name=item.name,
                            description=item.description,
                            data=item.data[:])
            list_columns.append(bp)
        bandsTable = DynamicTable(
            name=obj_old.bands.name,
            description=obj_old.bands.description,
            columns=list_columns,
            colnames=obj_old.bands.colnames
        )
        return DecompositionSeries(
            name=obj_old.name,
            data=obj_old.data[:],
            description=obj_old.description,
            metric=obj_old.metric,
            unit=obj_old.unit,
            rate=obj_old.rate,
            # source_timeseries=lfp,
            bands=bandsTable,
        )

    # Spectrum ----------------------------------------------------------------
    if type(obj_old) is Spectrum:
        file_elecs = nwb_new.electrodes
        nChannels = len(file_elecs['x'].data[:])
        elecs_region = file_elecs.create_region(
            name='electrodes',
            region=np.arange(nChannels).tolist(),
            description=''
        )
        return Spectrum(
            name=obj_old.name,
            frequencies=obj_old.frequencies[:],
            power=obj_old.power,
            electrodes=elecs_region
        )

    # Survey tables ------------------------------------------------------------
    if obj_old.neurodata_type == 'SurveyTable':
        if obj_old.name == 'nrs_survey_table':
            n_rows = len(obj_old['nrs_pain_intensity_rating'].data)
            for i in range(n_rows):
                nrs_survey_table.add_row(**{c: obj_old[c][i] for c in obj_old.colnames})
            return nrs_survey_table

        elif obj_old.name == 'vas_survey_table':
            n_rows = len(obj_old['vas_pain_intensity_rating'].data)
            for i in range(n_rows):
                vas_survey_table.add_row(**{c: obj_old[c][i] for c in obj_old.colnames})
            return vas_survey_table

        elif obj_old.name == 'mpq_survey_table':
            n_rows = len(obj_old['throbbing'].data)
            for i in range(n_rows):
                mpq_survey_table.add_row(**{c: obj_old[c][i] for c in obj_old.colnames})
            return mpq_survey_table



from PyQt5 import QtGui
from PyQt5.QtWidgets import (QGridLayout, QGroupBox, QLabel, QPushButton,
                             QVBoxLayout, QHBoxLayout, QCheckBox, QLineEdit,
                             QStyle, QFileDialog)
import pynwb


# Save to NWB file dialog ------------------------------------------------------
class SaveToNWBDialog(QtGui.QDialog):
    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.curr_nwbfile = self.parent.model.nwb
        self.value = False

        # New file path
        self.newfile = ''
        self.btn_newfile = QPushButton('Save to')
        self.btn_newfile.setIcon(self.style().standardIcon(QStyle.SP_DriveFDIcon))
        self.btn_newfile.clicked.connect(self.newfile_path)
        self.lin_newfile = QLineEdit('')

        self.panel_0 = QGridLayout()
        self.panel_0.addWidget(self.btn_newfile, 0, 0)
        self.panel_0.addWidget(self.lin_newfile, 0, 1)

        # Include raw data
        self.chk_raw = QCheckBox('Raw data')
        self.chk_raw.setChecked(False)
        self.chk_raw.setEnabled(False)
        for v in self.curr_nwbfile.acquisition.values():
            if v.__class__ is pynwb.ecephys.ElectricalSeries:
                self.chk_raw.setEnabled(True)
                self.chk_raw.setChecked(True)
                self.raw_name = v.name
        self.form_1 = QVBoxLayout()
        self.form_1.addWidget(self.chk_raw)

        # Include processing 'ecephys'
        self.checks_processing_ecephys = []
        if 'ecephys' in self.curr_nwbfile.processing:
            for k, v in self.curr_nwbfile.processing['ecephys'].data_interfaces.items():
                chk = QCheckBox(k)
                chk.setChecked(True)
                self.checks_processing_ecephys.append(chk)
                self.form_1.addWidget(chk)

        # Include processing 'behavior'
        self.checks_processing_behavior = []
        if 'behavior' in self.curr_nwbfile.processing:
            for k, v in self.curr_nwbfile.processing['behavior'].data_interfaces.items():
                if v.neurodata_type != 'SurveyTable':
                    chk = QCheckBox(k)
                    chk.setChecked(True)
                    self.checks_processing_behavior.append(chk)
                    self.form_1.addWidget(chk)
        self.panel_1 = QGroupBox('Include data:')
        self.panel_1.setLayout(self.form_1)

        # Include mic recording
        self.form_2 = QVBoxLayout()
        self.checks_mic = []
        for k, v in self.curr_nwbfile.acquisition.items():
            if v.__class__ is pynwb.ecephys.TimeSeries:
                chk = QCheckBox(k)
                chk.setChecked(True)
                self.checks_mic.append(chk)
                self.form_2.addWidget(chk)
        self.panel_2 = QGroupBox('Include microphone:')
        self.panel_2.setLayout(self.form_2)

        # Speaker stimuli
        self.form_3 = QVBoxLayout()
        self.checks_stimuli = []
        for k, v in self.curr_nwbfile.stimulus.items():
            chk = QCheckBox(k)
            chk.setChecked(True)
            self.checks_stimuli.append(chk)
            self.form_3.addWidget(chk)
        self.panel_3 = QGroupBox('Include stimuli:')
        self.panel_3.setLayout(self.form_3)

        self.form_4 = QVBoxLayout()

        # Surveys tables
        self.chk_surveys = QCheckBox('Surveys')
        self.chk_surveys.setChecked(False)
        if 'behavior' in self.curr_nwbfile.processing:
            if any([v for v in self.curr_nwbfile.processing['behavior'].data_interfaces.values()
                    if v.neurodata_type == 'SurveyTable']):
                self.form_4.addWidget(self.chk_surveys)
                self.chk_surveys.setChecked(True)

        # Include user-edited information
        self.chk_intervals = QCheckBox('Intervals')
        self.chk_intervals.setChecked(False)
        self.chk_annotations = QCheckBox('Annotations')
        self.chk_annotations.setChecked(False)
        self.form_4.addWidget(self.chk_intervals)
        self.form_4.addWidget(self.chk_annotations)

        self.panel_4 = QGroupBox('Include info:')
        self.panel_4.setLayout(self.form_4)

        # Buttons
        self.btn_save = QPushButton("Save")
        self.btn_save.clicked.connect(self.save)
        self.btn_cancel = QPushButton("Cancel")
        self.btn_cancel.clicked.connect(self.cancel)

        self.hbox_1 = QHBoxLayout()
        self.hbox_1.addWidget(self.btn_save)
        self.hbox_1.addWidget(self.btn_cancel)

        # Main Layout
        self.vbox_1 = QVBoxLayout()
        self.text0 = QLabel('New nwb file:')
        self.vbox_1.addWidget(self.text0)
        self.vbox_1.addLayout(self.panel_0)
        self.vbox_1.addWidget(QLabel(''))
        self.text1 = QLabel('Select what should be included in the new NWB file:')
        self.vbox_1.addWidget(self.text1)
        self.vbox_1.addWidget(self.panel_1)
        self.vbox_1.addWidget(self.panel_2)
        self.vbox_1.addWidget(self.panel_3)
        self.vbox_1.addWidget(self.panel_4)
        self.vbox_1.addLayout(self.hbox_1)

        self.setLayout(self.vbox_1)
        self.setWindowTitle('Save to NWB file')
        self.exec_()

    def newfile_path(self):
        filename, _ = QFileDialog.getSaveFileName(None, 'Save to file', '', "(*.nwb)")
        if filename:
            self.newfile = filename
            self.lin_newfile.setText(filename)

    def save(self):
        self.value = True
        self.make_cp_objs()
        self.accept()

    def cancel(self):
        self.value = False
        self.accept()

    def make_cp_objs(self):
        """Make dictionary of chosen objects to be passed to new file"""
        self.cp_objs = {
            'institution': True,
            'lab': True,
            'session': True,
            'devices': True,
            'electrode_groups': True,
            'electrodes': True,
            'epochs': True,
            'trials': True,
            'subject': True,
            'acquisition': [],
            'stimulus': [],
            'ecephys': [],
            'behavior': [],
            'surveys': False,
        }
        # Raw data
        if self.chk_raw.isChecked():
            self.cp_objs['acquisition'].append(self.raw_name)
        # Processed data
        for chk in self.checks_processing_ecephys:
            if chk.isChecked():
                self.cp_objs['ecephys'].append(chk.text())
        for chk in self.checks_processing_behavior:
            if chk.isChecked():
                self.cp_objs['behavior'].append(chk.text())
        # Mic recording
        for chk in self.checks_mic:
            if chk.isChecked():
                self.cp_objs['acquisition'].append(chk.text())
        # Speaker stimuli
        for chk in self.checks_stimuli:
            if chk.isChecked():
                self.cp_objs['stimulus'].append(chk.text())
        # Surveys
        if self.chk_surveys.isChecked():
            self.cp_objs.update({'surveys': True})
        # User-defined info
        if self.chk_intervals.isChecked():
            self.cp_objs.update({'intervals': True})



# -*- coding: utf-8 -*-
"""
Detect CV (consonant-vowel) pair events in speaker and microphone channels.
"""

# Third party libraries
import numpy as np
import scipy.signal as sgn
from process_nwb.resample import resample


def detect_events(speaker_data, mic_data=None, interval=None, dfact=30,
                  smooth_width=0.4, speaker_threshold=0.05, mic_threshold=0.05,
                  direction='both'):
    """
    Automatically detects events in audio signals.

    Parameters
    ----------
    speaker_data : 'pynwb.base.TimeSeries' object
        Object containing speaker data.
    mic_data : 'pynwb.base.TimeSeries' object
        Object containing microphone data.
    interval : list of floats
        Interval to be used [Start_bin, End_bin]. If 'None', the whole
        signal is used.
    dfact : float
        Downsampling factor. Default 30.
    smooth_width: float
        Width scale for median smoothing filter (default = .4, decent for CVs).
    speaker_threshold : float
        Sets threshold level for speaker.
    mic_threshold : float
        Sets threshold level for mic.
    direction : str
        'Up' detects events start times. 'Down' detects events stop times.
        'Both'
        detects both start and stop times.

    Returns
    -------
    speakerDS : 1D array of floats
        Downsampled speaker signal.
    speakerEventDS : 1D array of floats
        Event times for speaker signal.
    speakerFilt : 1D array of floats
        Filtered speaker signal.
    micDS : 1D array of floats
        Downsampled microphone signal.
    micEventDS : 1D array of floats
        Event times for microphone signal.
    micFilt : 1D array of floats
        Filtered microphone signal.
    """

    # Downsampling Speaker ---------------------------------------------------

    speakerDS, speakerEventDS, speakerFilt = None, None, None

    if speaker_data is not None:
        if interval is None:
            X = speaker_data.data[:]
        else:
            X = speaker_data.data[interval[0]:interval[1]]
        fs = speaker_data.rate  # sampling rate
        ds = fs / dfact

        # Pad zeros to make signal length a power of 2, improves performance
        nBins = X.shape[0]
        extraBins = 2 ** (np.ceil(np.log2(nBins)).astype('int')) - nBins
        extraZeros = np.zeros(extraBins)
        X = np.append(X, extraZeros)
        speakerDS = resample(X, ds, fs)

        # Remove excess bins (because of zero padding on previous step)
        excessBins = int(np.ceil(extraBins * ds / fs))
        speakerDS = speakerDS[0:-excessBins]

        # Kernel size must be an odd number
        speakerFilt = sgn.medfilt(
            volume=np.diff(np.append(speakerDS, speakerDS[-1])) ** 2,
            kernel_size=int((smooth_width * ds // 2) * 2 + 1)
        )

        # Normalize the filtered signal.
        speakerFilt /= np.max(np.abs(speakerFilt))

        # Find threshold crossing times
        stimBinsDS = threshcross(speakerFilt, speaker_threshold, direction)

        # Remove events that have a duration less than 0.1 s.
        speaker_events = stimBinsDS.reshape((-1, 2))
        rem_ind = np.where((speaker_events[:, 1] - speaker_events[:, 0]) < ds * 0.1)[0]
        speaker_events = np.delete(speaker_events, rem_ind, axis=0)
        stimBinsDS = speaker_events.reshape((-1))

        # Transform bins to time
        if interval is None:
            speakerEventDS = (stimBinsDS / ds)
        else:
            speakerEventDS = (stimBinsDS / ds) + (interval[0] / fs)

    # Downsampling Mic -------------------------------------------------------

    micDS, micEventDS, micFilt = None, None, None

    if mic_data is not None:

        if interval is None:
            X = mic_data.data[:]
        else:
            X = mic_data.data[interval[0]:interval[1]]
        fs = mic_data.rate  # sampling rate
        ds = fs / dfact

        # Pad zeros to make signal length a power of 2, improves performance
        nBins = X.shape[0]
        extraBins = 2 ** (np.ceil(np.log2(nBins)).astype('int')) - nBins
        extraZeros = np.zeros(extraBins)
        X = np.append(X, extraZeros)
        micDS = resample(X, ds, fs)

        # Remove excess bins (because of zero padding on previous step)
        excessBins = int(np.ceil(extraBins * ds / fs))
        micDS = micDS[0:-excessBins]

        # Remove mic response to speaker
        micDS[np.where(speakerFilt > speaker_threshold)[0]] = 0
        micFilt = sgn.medfilt(volume=np.diff(np.append(micDS, micDS[-1])) ** 2,
                              kernel_size=int(
                                  (smooth_width * ds // 2) * 2 + 1))

        # Normalize the filtered signal.
        micFilt /= np.max(np.abs(micFilt))

        # Find threshold crossing times
        micBinsDS = threshcross(micFilt, mic_threshold, direction)

        # Remove events that have a duration less than 0.1 s.
        mic_events = micBinsDS.reshape((-1, 2))
        rem_ind = np.where((mic_events[:, 1] - mic_events[:, 0]) < ds * 0.1)[0]
        mic_events = np.delete(mic_events, rem_ind, axis=0)
        micBinsDS = mic_events.reshape((-1))

        # Transform bins to time
        if interval is None:
            micEventDS = (micBinsDS / ds)
        else:
            micEventDS = (micBinsDS / ds) + (interval[0] / fs)

    return speakerDS, speakerEventDS, speakerFilt, micDS, micEventDS, micFilt


def threshcross(data, threshold=0, direction='up'):
    """
    Outputs the indices where the signal crossed the threshold.

    Parameters
    ----------
    data : array of floats
        Numpy array of floats, containing signal.
    threshold : float
        Value of threshold.
    direction : str
        Defines the direction of cross detected: 'up', 'down', or 'both'.
        With 'both', it will check to make sure that up and down crosses are
        detected. In other words,

    Returns
    -------
    out : array
        Array with indices where data crossed threshold.
    """

    # Find crosses
    over = (data >= threshold).astype('int')
    cross = np.append(False, np.diff(over))

    if direction == 'up':
        out = np.where(cross == 1)[0]
    elif direction == 'down':
        out = np.where(cross == -1)[0]
    elif direction == 'both':
        cross_nonzero = np.where(cross != 0)[0]

        events = []
        for i in range(len(cross_nonzero) - 1):

            # Skip to the next ind if this one was already recorded.
            if cross_nonzero[i] in events:
                continue

            if (cross[cross_nonzero[i]] == 1) and (
                    cross[cross_nonzero[i + 1]] == -1):
                events.append(cross_nonzero[i])
                events.append(cross_nonzero[i + 1])

        out = np.array(events)

    return out



from ndx_hierarchical_behavioral_data.transcription_io import timitsounds_df, timitsounds_converter
from ndx_hierarchical_behavioral_data.mocha_io import mocha_df, mocha_re_df, mocha_converter
from ndx_hierarchical_behavioral_data.text_grid_io import textgriddf_reader, textgriddf_df, textgriddf_converter


def add_transcription_data(nwbfile, path_transcription, tr_type, subject_id=None,
                           session_id=None):
    """
    Add transcription data to nwb file

    Parameters:
    -----------
    nwbfile : nwbfile
    path_transcription : path
    tr_type : str
        Transcriptions data type: timitsounds, textgrid or mocha
    subject_id : str
        Used for Mocha
    session_id : str
        Used for Mocha
    """

    if tr_type == 'timitsounds':
        phonemes, syllables, words, sentences = get_timitsounds(path_transcription=path_transcription)
    elif tr_type == 'mocha':
        phonemes, syllables, words, sentences = get_mocha(
            path_transcription=path_transcription,
            subject_id=subject_id,
            session_id=session_id
        )
        if sentences is None:  # If no transcription data for given session
            return None
    elif tr_type == 'textgrid':
        phonemes, syllables, words = None, None, None
        sentences = get_textgrid(path_transcription=path_transcription)
    else:
        raise TypeError('Invalid transcription type')

    # Create behavioral processing module, if not existent
    if 'behavior' not in nwbfile.processing:
        nwbfile.create_processing_module(
            name='behavior',
            description='behavioral data'
        )

    # Add transcription tables to behavioral processing module
    if phonemes is not None:
        nwbfile.processing['behavior'].add(phonemes)
    if syllables is not None:
        nwbfile.processing['behavior'].add(syllables)
    if words is not None:
        nwbfile.processing['behavior'].add(words)
    if sentences is not None:
        nwbfile.processing['behavior'].add(sentences)

    return nwbfile


def get_textgrid(path_transcription):
    """Get data from TextGrid file"""
    data = textgriddf_reader(path_file=path_transcription)
    text_df = textgriddf_df(data, item_no=2)
    sentences = textgriddf_converter(text_df)

    return sentences


def get_mocha(path_transcription, subject_id, session_id):
    """Get data from Mocha directory"""
    phoneme_data, syllable_data, word_data, sentences_data = mocha_df(path_to_files=path_transcription)
    re_phoneme_data, re_syllable_data, re_word_data, re_sentence_data = mocha_re_df(
        phoneme_data=phoneme_data,
        syllable_data=syllable_data,
        word_data=word_data,
        sentences_data=sentences_data,
        subject_id=subject_id,  # 'EC118'
        session_id=session_id,  # 'B6'
        trial_id='...'
    )
    if re_sentence_data.shape[0] == 0:
        return None, None, None, None
    else:
        phonemes, syllables, words, sentences = mocha_converter(
            re_phoneme_data=re_phoneme_data,
            re_syllable_data=re_syllable_data,
            re_word_data=re_word_data,
            re_sentence_data=re_sentence_data
        )
        return phonemes, syllables, words, sentences


def get_timitsounds(path_transcription):
    """Get data from TimitSounds directory"""
    phonemes_data, syllables_data, words_data, sentences_data, pitch_data, formant_data, intensity_data = timitsounds_df(path_to_files=path_transcription)
    phonemes, syllables, words, sentences, pitch_ts, formant_ts, intensity_ts = timitsounds_converter(
        phonemes_data=phonemes_data,
        syllables_data=syllables_data,
        words_data=words_data,
        sentences_data=sentences_data,
        pitch_data=pitch_data,
        formant_data=formant_data,
        intensity_data=intensity_data
    )
    return phonemes, syllables, words, sentences



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

# Get the long description from the README file
with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='ecogvis',
    version='1.2.1',
    description='Timeseries visualizer for Electrocorticography (ECoG) signals stored in NWB files.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Luiz Tauffer and Ben Dichter',
    email='ben.dichter@gmail.com',
    url='https://github.com/catalystneuro/ecogVIS',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'PyQt5',
        'matplotlib',
        'cycler',
        'scipy',
        'numpy',
        'h5py',
        'pyqtgraph',
        'pandas',
        'pynwb>=1.1.2',
        'PyYAML',
        'ndx-ecog',
        'ndx-spectrum',
        'ndx-survey-data',
        'ndx-bipolar-scheme',
        'tqdm',
        'pyopengl',
        'process_nwb',
        'PyQtWebEngine',
        'ndx-icephys-meta',
        'ndx-hierarchical-behavioral-data'],
    entry_points={
        'console_scripts': ['ecogvis=ecogvis.ecogvis:cmd_line_shortcut'],
    }
)



from PyQt5 import QtGui, QtCore
from PyQt5.QtWidgets import (QGridLayout, QGroupBox, QLineEdit, QHBoxLayout,
                             QWidget, QLabel, QPushButton, QVBoxLayout,
                             QComboBox, QScrollArea, QMainWindow)
import pyqtgraph as pg
from ecogvis.functions.misc_dialogs import SelectChannelsDialog
from .FS_colorLUT import get_lut
import numpy as np

default_interval = 'TimeIntervals_speaker'
default_alignment = 'start_time'


# Creates Event-Related Potential dialog ---------------------------------------
class ERPDialog(QMainWindow):
    def __init__(self, parent):
        super().__init__()
        self.setWindowTitle('Event-Related Potentials')
        self.resize(1300, 600)

        self.parent = parent
        self.nCols = 16
        self.grid_order = np.arange(256)
        self.transparent = []
        self.Yscale = {}

        self.all_regions = np.unique(self.parent.model.all_regions)
        self.regions_mask = np.ones(len(self.all_regions))

        self.source = self.parent.model.nwb.processing['ecephys'].data_interfaces['high_gamma'].data
        self.fs = self.parent.model.nwb.processing['ecephys'].data_interfaces['high_gamma'].rate
        self.electrodes = self.parent.model.nwb.processing['ecephys'].data_interfaces['high_gamma'].electrodes
        self.maxTime = self.source.shape[0] / self.fs

        # Make intervals dictionary, e.g. self.intervals['TimeIntervals_speaker']['start_time']['times'] = data[:]
        self.intervals = {}
        for interval_type, interval_obj in self.parent.model.nwb.intervals.items():
            self.intervals[interval_type] = {}
            for alignment_type, obj in zip(interval_obj.colnames, interval_obj.columns):
                times = obj.data[:]
                # Get only reference times smaller than the main signal duration
                times = times[times < self.maxTime]
                self.intervals[interval_type][alignment_type] = {}
                self.intervals[interval_type][alignment_type]['times'] = times
                self.intervals[interval_type][alignment_type]['Y_mean'] = {}
                self.intervals[interval_type][alignment_type]['Y_sem'] = {}
        self.X = []

        self.interval_sel = default_interval
        self.alignment_sel = default_alignment

        # Left panel
        self.push0_0 = QPushButton('Draw ERP')
        self.push0_0.clicked.connect(self.set_elec_group)
        label0 = QLabel('Group:')
        self.combo0 = QComboBox()
        self.find_groups()
        self.combo0.activated.connect(self.set_elec_group)
        label1 = QLabel('Alignment:')
        self.combo_interval = QComboBox()
        self.combo_interval.activated.connect(self.set_interval)
        self.combo_alignment = QComboBox()
        self.combo_alignment.activated.connect(self.set_alignment)
        self.find_intervals()

        label2 = QLabel('Width (sec):')
        self.qline2 = QLineEdit('2')
        self.qline2.returnPressed.connect(self.set_width)
        label3 = QLabel('Y scale:')
        self.combo1 = QComboBox()
        self.combo1.addItem('individual')
        self.combo1.addItem('global max')
        self.combo1.addItem('global std')
        self.combo1.activated.connect(self.scale_plots)
        self.push2_0 = QPushButton('Significant')
        self.push2_0.setCheckable(True)
        self.push2_0.setChecked(False)
        self.push3_0 = QPushButton('Brain areas')
        self.push3_0.clicked.connect(self.areas_select)
        self.push4_0 = QPushButton('Save image')
        self.push4_0.clicked.connect(self.save_image)
        label4 = QLabel('Rotate grid:')
        self.push5_0 = QPushButton('90°')
        self.push5_0.clicked.connect(lambda: self.rearrange_grid(90))
        self.push5_0.setToolTip('Counter-clockwise')
        self.push5_1 = QPushButton('-90°')
        self.push5_1.clicked.connect(lambda: self.rearrange_grid(-90))
        self.push5_1.setToolTip('Clockwise')
        self.push5_2 = QPushButton('T')
        self.push5_2.clicked.connect(lambda: self.rearrange_grid('T'))
        self.push5_2.setToolTip('Transpose')
        label5 = QLabel('Rearrange grid:')
        self.push5_3 = QPushButton('L-R')
        self.push5_3.clicked.connect(lambda: self.rearrange_grid('FLR'))
        self.push5_3.setToolTip('Flip Left-Right')
        self.push5_4 = QPushButton('U-D')
        self.push5_4.clicked.connect(lambda: self.rearrange_grid('FUD'))
        self.push5_4.setToolTip('Flip Up-Down')
        self.push5_5 = QPushButton('2FL')
        self.push5_5.clicked.connect(lambda: self.rearrange_grid('2FL'))
        self.push5_5.setToolTip('Double flip')

        self.qline2.setEnabled(False)
        self.combo1.setEnabled(False)
        self.push2_0.setEnabled(False)
        self.push3_0.setEnabled(False)
        self.push4_0.setEnabled(False)
        self.push5_0.setEnabled(False)
        self.push5_1.setEnabled(False)
        self.push5_2.setEnabled(False)
        self.push5_3.setEnabled(False)
        self.push5_4.setEnabled(False)
        self.push5_5.setEnabled(False)

        grid0 = QGridLayout()
        grid0.addWidget(label0, 0, 0, 1, 2)
        grid0.addWidget(self.combo0, 0, 2, 1, 4)
        grid0.addWidget(label1, 1, 0, 1, 6)
        grid0.addWidget(self.combo_interval, 2, 0, 1, 6)
        grid0.addWidget(self.combo_alignment, 3, 0, 1, 6)
        grid0.addWidget(QHLine(), 4, 0, 1, 6)
        grid0.addWidget(label2, 5, 0, 1, 6)
        grid0.addWidget(self.qline2, 6, 0, 1, 6)
        grid0.addWidget(QHLine(), 7, 0, 1, 6)
        grid0.addWidget(label3, 8, 0, 1, 6)
        grid0.addWidget(self.combo1, 9, 0, 1, 6)
        grid0.addWidget(QHLine(), 10, 0, 1, 6)
        grid0.addWidget(self.push2_0, 11, 0, 1, 6)
        grid0.addWidget(self.push3_0, 12, 0, 1, 6)
        grid0.addWidget(self.push4_0, 13, 0, 1, 6)
        grid0.addWidget(QHLine(), 14, 0, 1, 6)
        grid0.addWidget(label4, 15, 0, 1, 6)
        grid0.addWidget(self.push5_0, 16, 0, 1, 2)
        grid0.addWidget(self.push5_1, 16, 2, 1, 2)
        grid0.addWidget(self.push5_2, 16, 4, 1, 2)
        grid0.addWidget(label5, 17, 0, 1, 6)
        grid0.addWidget(self.push5_3, 18, 0, 1, 2)
        grid0.addWidget(self.push5_4, 18, 2, 1, 2)
        grid0.addWidget(self.push5_5, 18, 4, 1, 2)
        grid0.setAlignment(QtCore.Qt.AlignTop)

        panel0 = QGroupBox('Controls:')
        panel0.setFixedWidth(180)
        panel0.setLayout(grid0)

        self.leftbox = QVBoxLayout()
        self.leftbox.addWidget(self.push0_0)
        self.leftbox.addWidget(panel0)

        # Right panel
        self.win = pg.GraphicsLayoutWidget()
        self.win.resize(1020, 1020)
        background_color = self.palette().color(QtGui.QPalette.Background)
        self.win.setBackground(background_color)
        # this is to avoid the error:
        # RuntimeError: wrapped C/C++ object of type GraphicsScene has been deleted
        vb = CustomViewBox(self, 0)
        p = self.win.addPlot(0, 0, viewBox=vb)
        p.hideAxis('left')
        p.hideAxis('bottom')
        # Scroll Area Properties
        self.scroll = QScrollArea()
        self.scroll.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scroll.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scroll.setWidgetResizable(False)
        self.scroll.setWidget(self.win)

        self.centralwidget = QWidget()
        self.setCentralWidget(self.centralwidget)
        self.hbox = QHBoxLayout(self.centralwidget)
        self.hbox.addLayout(self.leftbox)    # add panels first
        self.hbox.addWidget(self.scroll)
        self.show()

        self.set_elec_group()
        self.set_interval(sel=self.interval_sel)

    def find_groups(self):
        """Find electrodes groups present in current file."""
        elec_groups = list(self.parent.model.nwb.electrode_groups.keys())
        for grp in elec_groups:
            self.combo0.addItem(grp)

    def find_intervals(self):
        intervals = list(self.intervals.keys())
        for ivl in intervals:
            self.combo_interval.addItem(ivl)

    def find_alignments(self, starting_sel=default_alignment):
        """Find alignment types for current interval type"""
        self.combo_alignment.clear()
        ivl = self.combo_interval.currentText()
        alignments = list(self.intervals[ivl].keys())
        for aln in alignments:
            self.combo_alignment.addItem(aln)

        self.set_alignment(sel=starting_sel)

    def set_elec_group(self):
        """Sets electrodes group to be plotted, resizes plot grid."""
        self.elec_group = self.combo0.currentText()
        self.grid_order = np.where(self.electrodes.table['group_name'].data[:] == self.elec_group)[0]
        self.nElecs = len(self.grid_order)
        self.nCols = 16
        self.nRows = int(np.ceil(self.nElecs / self.nCols))
        self.set_grid()
        self.draw_erp()

    def set_interval(self, sel=None):
        if isinstance(sel, int):
            sel = self.combo_interval.itemText(sel)
        if sel is not None:
            index = self.combo_interval.findText(sel, QtCore.Qt.MatchFixedString)
            if index >= 0:
                self.combo_interval.setCurrentIndex(index)

        sel = self.combo_interval.currentText()
        self.interval_sel = sel
        self.find_alignments(starting_sel=self.alignment_sel)

    def set_alignment(self, sel=None):
        if isinstance(sel, int):
            sel = self.combo_alignment.itemText(sel)
        if sel is not None:
            index = self.combo_alignment.findText(sel, QtCore.Qt.MatchFixedString)
            if index >= 0:
                self.combo_alignment.setCurrentIndex(index)

        sel = self.combo_alignment.currentText()
        self.alignment_sel = sel
        self.draw_erp()

    def set_grid(self):
        # remove previous items
        for i in np.arange(16):
            for j in np.arange(16):
                it = self.win.getItem(i, j)
                if it is not None:
                    self.win.removeItem(it)
        for j in np.arange(self.nCols):
            self.win.ci.layout.setColumnFixedWidth(j, 60)
            self.win.ci.layout.setColumnSpacing(j, 3)
        for i in np.arange(self.nRows):
            self.win.ci.layout.setRowFixedHeight(i, 60)
            self.win.ci.layout.setRowSpacing(i, 3)

    def set_width(self):
        for interval_type, interval_dict in self.intervals.items():
            for alignment_type, alignment_dict in interval_dict.items():
                alignment_dict['Y_mean'] = {}
                alignment_dict['Y_sem'] = {}
        self.X = []
        self.draw_erp()

    def rearrange_grid(self, angle):
        grid = self.grid_order.reshape(-1, self.nCols)  # re-arranges as 2D array
        # 90 degrees clockwise
        if angle == -90:
            grid = np.rot90(grid, axes=(1, 0))
            if self.nRows != self.nCols:
                aux = np.copy(self.nRows)
                self.nRows = np.copy(self.nCols)
                self.nCols = aux
        # 90 degrees conter-clockwise
        elif angle == 90:
            grid = np.rot90(grid, axes=(0, 1))
            if self.nRows != self.nCols:
                aux = np.copy(self.nRows)
                self.nRows = np.copy(self.nCols)
                self.nCols = aux
        # Transpose
        elif angle == 'T':
            grid = grid.T
            if self.nRows != self.nCols:
                aux = np.copy(self.nRows)
                self.nRows = np.copy(self.nCols)
                self.nCols = aux
        # Flip left-right
        elif angle == 'FLR':
            grid = np.flip(grid, 1)
        # Flip up-down
        elif angle == 'FUD':
            grid = np.flip(grid, 0)
        # Double flip
        elif angle == '2FL':
            grid = np.flip(grid, 1)
            grid = np.flip(grid, 0)
        self.grid_order = grid.flatten()    # re-arranges as 1D array
        self.draw_erp()

    def save_image(self):
        p = self.win.getItem(row=0, col=0)
        self.win.sceneObj.contextMenuItem = p
        self.win.sceneObj.showExportDialog()

    def scale_plots(self):
        for ind, ch in enumerate(self.grid_order):
            row = np.floor(ind / self.nCols)
            col = ind % self.nCols
            p = self.win.getItem(row=row, col=col)
            if p is None:
                return
            else:
                curr_txt = self.combo1.currentText()
                if curr_txt != 'individual':
                    p.setYRange(self.Yscale[curr_txt][0], self.Yscale[curr_txt][1])
                else:
                    yrng = max(abs(self.intervals[self.interval_sel][self.alignment_sel]['Y_mean'][str(ch)]))
                    p.setYRange(-yrng, yrng)

    def get_erp(self, ch):
        try:
            return self.intervals[self.interval_sel][self.alignment_sel]['Y_mean'][str(ch)], \
                   self.intervals[self.interval_sel][self.alignment_sel]['Y_sem'][str(ch)], \
                   self.X
        except:
            Y_mean, Y_sem, X = self.calc_erp(ch=ch)
            self.intervals[self.interval_sel][self.alignment_sel]['Y_mean'][str(ch)] = Y_mean
            self.intervals[self.interval_sel][self.alignment_sel]['Y_sem'][str(ch)] = Y_sem
            self.X = X
            return self.intervals[self.interval_sel][self.alignment_sel]['Y_mean'][str(ch)], \
                   self.intervals[self.interval_sel][self.alignment_sel]['Y_sem'][str(ch)], \
                   self.X

    def calc_erp(self, ch):
        ref_times = self.intervals[self.interval_sel][self.alignment_sel]['times']
        ref_bins = (ref_times * self.fs).astype('int')
        nBinsTr = int(float(self.qline2.text()) * self.fs / 2)
        start_bins = ref_bins - nBinsTr
        stop_bins = ref_bins + nBinsTr
        nTrials = len(ref_times)
        Y = np.zeros((nTrials, 2 * nBinsTr)) + np.nan
        for tr in np.arange(nTrials):
            # Get only trials with enough data points
            if self.source[start_bins[tr]:stop_bins[tr], ch].shape[0] == Y.shape[1]:
                Y[tr, :] = self.source[start_bins[tr]:stop_bins[tr], ch]
        Y_mean = np.nanmean(Y, 0)
        Y_sem = np.nanstd(Y, 0) / np.sqrt(Y.shape[0])
        X = np.arange(0, 2 * nBinsTr) / self.fs
        return Y_mean, Y_sem, X

    def draw_erp(self):
        self.qline2.setEnabled(True)
        self.combo1.setEnabled(True)
        self.push3_0.setEnabled(True)
        self.push4_0.setEnabled(True)
        if self.nElecs % 16 == 0:
            self.push5_0.setEnabled(True)
            self.push5_1.setEnabled(True)
            self.push5_2.setEnabled(True)
            self.push5_3.setEnabled(True)
            self.push5_4.setEnabled(True)
            self.push5_5.setEnabled(True)
        else:
            self.push5_0.setEnabled(False)
            self.push5_1.setEnabled(False)
            self.push5_2.setEnabled(False)
            self.push5_3.setEnabled(False)
            self.push5_4.setEnabled(False)
            self.push5_5.setEnabled(False)
        self.combo1.setCurrentIndex(self.combo1.findText('individual'))
        self.set_grid()
        cmap = get_lut()
        ymin, ymax = 0, 0
        ystd = 0
        for ind, ch in enumerate(self.grid_order):
            if ch in self.transparent:  # if it should be made transparent
                elem_alpha = 30
            else:
                elem_alpha = 255
            Y_mean, Y_sem, X = self.get_erp(ch=ch)
            dc = np.mean(Y_mean)
            Y_mean -= dc
            ymax = max(max(Y_mean), ymax)
            ymin = min(min(Y_mean), ymin)
            ystd = max(np.std(Y_mean), ystd)
            # Include items
            row = np.floor(ind / self.nCols).astype('int')
            col = int(ind % self.nCols)
            p = self.win.getItem(row=row, col=col)
            if p is None:
                vb = CustomViewBox(self, ch)
                p = self.win.addPlot(row=row, col=col, viewBox=vb)
            p.hideAxis('left')
            p.hideAxis('bottom')
            p.clear()
            p.setMouseEnabled(x=False, y=False)
            p.setToolTip('Ch ' + str(ch + 1) + '\n' + str(self.parent.model.nwb.electrodes['location'][ch]))
            # Background
            loc = 'ctx-lh-' + self.parent.model.nwb.electrodes['location'][ch]
            vb = p.getViewBox()
            color = tuple(cmap.get(loc, cmap['Unknown']))
            vb.setBackgroundColor(
                (*color, min(elem_alpha, 70)))  # append alpha to color tuple
            # Main plots
            mean = p.plot(x=X, y=Y_mean, pen=pg.mkPen((50, 50, 50, min(elem_alpha, 255)), width=1.))
            semp = p.plot(x=X, y=Y_mean + Y_sem, pen=pg.mkPen((100, 100, 100, min(elem_alpha, 100)), width=.1))
            semm = p.plot(x=X, y=Y_mean - Y_sem, pen=pg.mkPen((100, 100, 100, min(elem_alpha, 100)), width=.1))
            fill = pg.FillBetweenItem(semm, semp, pg.mkBrush(100, 100, 100, min(elem_alpha, 100)))
            p.addItem(fill)
            p.hideButtons()
            p.setXRange(X[0], X[-1])
            yrng = max(abs(Y_mean))
            p.setYRange(-yrng, yrng)
            xref = [X[int(len(X) / 2)], X[int(len(X) / 2)]]
            yref = [-1000 * yrng, 1000 * yrng]
            p.plot(x=xref, y=yref, pen=(0, 0, 0, min(elem_alpha, 255)))    # reference mark
            p.plot(x=X, y=np.zeros(len(X)), pen=(0, 0, 0, min(elem_alpha, 255)))  # Zero line
            # Axis control
            left = p.getAxis('left')
            left.setStyle(showValues=False)
            left.setTicks([])
            bottom = p.getAxis('bottom')
            bottom.setStyle(showValues=False)
            bottom.setTicks([])
        # store scale limits
        self.Yscale['global max'] = [ymin, ymax]
        self.Yscale['global std'] = [-ystd, ystd]

    def areas_select(self):
        # Dialog to choose channels from specific brain regions
        w = SelectChannelsDialog(self.all_regions, self.regions_mask)
        self.transparent = []
        for ind, ch in enumerate(self.grid_order):
            loc = self.parent.model.nwb.electrodes['location'][ch]
            if loc not in w.choices:
                self.transparent.append(ch)
        self.draw_erp()


# Viewbox for ERP plots --------------------------------------------------------
class CustomViewBox(pg.ViewBox):
    def __init__(self, parent, ch):
        pg.ViewBox.__init__(self)
        self.parent = parent
        self.ch = ch

    def mouseDoubleClickEvent(self, ev):
        IndividualERPDialog(self)


# Individual Event-Related Potential dialog ------------------------------------
class IndividualERPDialog(QtGui.QDialog):
    def __init__(self, parent):
        super().__init__()
        # Enable antialiasing for prettier plots
        pg.setConfigOptions(antialias=True)

        self.parent = parent
        self.ch = parent.ch
        self.alignment = 'start_time'
        self.interval_type = 'speaker'
        self.source = self.parent.parent.source
        self.fs = self.parent.parent.fs
        self.intervals = self.parent.parent.intervals

        self.interval_sel = self.parent.parent.interval_sel
        self.alignment_sel = self.parent.parent.alignment_sel

        # Left panel
        label1 = QLabel('Alignment:')
        self.combo_interval = QComboBox()
        self.combo_interval.activated.connect(self.set_interval)
        self.combo_alignment = QComboBox()
        self.combo_alignment.activated.connect(self.set_alignment)
        self.find_intervals()

        label2 = QLabel('Width (sec):')
        self.qline2 = QLineEdit('2')
        self.qline2.returnPressed.connect(self.draw_erp)

        grid0 = QGridLayout()
        grid0.addWidget(label1, 0, 0, 1, 2)
        grid0.addWidget(self.combo_interval, 1, 0, 1, 6)
        grid0.addWidget(self.combo_alignment, 2, 0, 1, 6)
        grid0.addWidget(label2, 3, 0, 1, 1)
        grid0.addWidget(self.qline2, 3, 1, 1, 1)
        grid0.setAlignment(QtCore.Qt.AlignTop)
        panel0 = QGroupBox('Controls:')
        panel0.setFixedWidth(180)
        panel0.setLayout(grid0)
        self.leftbox = QVBoxLayout()
        self.leftbox.addWidget(panel0)

        # Right panel
        self.win = pg.GraphicsLayoutWidget()
        self.win.resize(900, 600)
        self.win.setBackground('w')
        self.hbox = QHBoxLayout()
        # self.hbox.addWidget(panel0)
        self.hbox.addLayout(self.leftbox)
        self.hbox.addWidget(self.win)
        self.setLayout(self.hbox)
        self.setWindowTitle('Individual Event-Related Potential - Ch ' + str(self.ch + 1))
        self.resize(900, 600)

        self.set_interval(sel=self.interval_sel)

        self.draw_erp()
        self.exec_()

    def find_intervals(self):
        intervals = list(self.intervals.keys())
        for ivl in intervals:
            self.combo_interval.addItem(ivl)

    def find_alignments(self, starting_sel=default_alignment):
        """Find alignment types for current interval type"""
        self.combo_alignment.clear()
        ivl = self.combo_interval.currentText()
        alignments = list(self.intervals[ivl].keys())
        for aln in alignments:
            self.combo_alignment.addItem(aln)
        self.set_alignment(sel=starting_sel)

    def set_interval(self, sel=None):
        if isinstance(sel, int):
            sel = self.combo_interval.itemText(sel)
        if sel is not None:
            index = self.combo_interval.findText(sel, QtCore.Qt.MatchFixedString)
            if index >= 0:
                self.combo_interval.setCurrentIndex(index)
        sel = self.combo_interval.currentText()
        self.interval_sel = sel
        self.find_alignments(starting_sel=self.alignment_sel)

    def set_alignment(self, sel=None):
        if isinstance(sel, int):
            sel = self.combo_alignment.itemText(sel)
        if sel is not None:
            index = self.combo_alignment.findText(sel, QtCore.Qt.MatchFixedString)
            if index >= 0:
                self.combo_alignment.setCurrentIndex(index)
        sel = self.combo_alignment.currentText()
        self.alignment_sel = sel
        self.draw_erp()

    def get_erp(self, ch):
        try:
            return self.intervals[self.interval_sel][self.alignment_sel]['Y_mean'][str(ch)], \
                   self.intervals[self.interval_sel][self.alignment_sel]['Y_sem'][str(ch)], \
                   self.X
        except:
            Y_mean, Y_sem, X = self.calc_erp(ch=ch)
            self.intervals[self.interval_sel][self.alignment_sel]['Y_mean'][str(ch)] = Y_mean
            self.intervals[self.interval_sel][self.alignment_sel]['Y_sem'][str(ch)] = Y_sem
            self.X = X
            return self.intervals[self.interval_sel][self.alignment_sel]['Y_mean'][str(ch)], \
                   self.intervals[self.interval_sel][self.alignment_sel]['Y_sem'][str(ch)], \
                   self.X

    def calc_erp(self, ch):
        ref_times = self.intervals[self.interval_sel][self.alignment_sel]['times']
        ref_bins = (ref_times * self.fs).astype('int')
        nBinsTr = int(float(self.qline2.text()) * self.fs / 2)
        start_bins = ref_bins - nBinsTr
        stop_bins = ref_bins + nBinsTr
        nTrials = len(ref_times)
        Y = np.zeros((nTrials, 2 * nBinsTr)) + np.nan
        for tr in np.arange(nTrials):
            # Get only trials with enough data points
            if self.source[start_bins[tr]:stop_bins[tr], ch].shape[0] == Y.shape[1]:
                Y[tr, :] = self.source[start_bins[tr]:stop_bins[tr], ch]
        Y_mean = np.nanmean(Y, 0)
        Y_sem = np.nanstd(Y, 0) / np.sqrt(Y.shape[0])
        X = np.arange(0, 2 * nBinsTr) / self.fs
        X -= X[-1] / 2
        return Y_mean, Y_sem, X

    def draw_erp(self):
        cmap = get_lut()
        Y_mean, Y_sem, X = self.calc_erp(ch=self.ch)
        dc = np.mean(Y_mean)
        Y_mean -= dc
        p = self.win.getItem(row=0, col=0)
        if p is None:
            p = self.win.addPlot(row=0, col=0)
        p.clear()
        p.setMouseEnabled(x=False, y=True)
        # Background color
        loc = 'ctx-lh-' + self.parent.parent.parent.model.nwb.electrodes['location'][self.ch]
        vb = p.getViewBox()
        color = tuple(cmap.get(loc, cmap['Unknown']))
        vb.setBackgroundColor((*color, 70))  # append alpha to color tuple
        vb.border = pg.mkPen(color='w')
        # Main plots
        mean = p.plot(x=X, y=Y_mean, pen=pg.mkPen((60, 60, 60), width=2.))
        semp = p.plot(x=X, y=Y_mean + Y_sem, pen=pg.mkPen((100, 100, 100, 100), width=.1))
        semm = p.plot(x=X, y=Y_mean - Y_sem, pen=pg.mkPen((100, 100, 100, 100), width=.1))
        fill = pg.FillBetweenItem(semm, semp, pg.mkBrush(100, 100, 100, 100))
        p.addItem(fill)
        p.hideButtons()
        p.setXRange(X[0], X[-1])
        p.setYRange(min(Y_mean), max(Y_mean))
        xref = [X[int(len(X) / 2)], X[int(len(X) / 2)]]
        yref = [-1000, 1000]
        p.plot(x=xref, y=yref, pen=(0, 0, 0))    # reference mark
        p.plot(x=X, y=np.zeros(len(X)), pen=(0, 0, 0))  # Zero line
        # Axis control
        left = p.getAxis('left')
        left.setStyle(showValues=False)
        left.setTicks([])
        bottom = p.getAxis('bottom')
        bottom.setStyle(showValues=True)
        p.setLabel('bottom', 'Time', units='sec')


# Gray line for visual separation of buttons -----------------------------------
class QHLine(QtGui.QFrame):
    def __init__(self):
        super().__init__()
        self.setFrameShape(QtGui.QFrame.HLine)
        self.setFrameShadow(QtGui.QFrame.Sunken)



from ndx_survey_data import survey_definitions
from scipy.io import loadmat
from importlib import reload
import math


def add_survey_data(nwbfile, path_survey_file):
    """
    Add survey data to nwb file

    Parameters:
    -----------
    nwbfile: nwbfile
    path_survey_fila: path
        Path to .mat file containing survey data
    """

    # Reload definitions
    reload(survey_definitions)

    survey_dict = loadmat(path_survey_file)
    survey_data = survey_dict['redcap_painscores']
    survey_dt = survey_dict['redcap_datetimes'][:, 0]

    # Mpq data values
    def mpq_values(val):
        if math.isnan(val):
            return 'NaN'
        elif int(val) == 1:
            return 'mild'
        elif int(val) == 2:
            return 'moderate'
        elif int(val) == 3:
            return 'severe'

    # NRS and VAS data values
    def nrs_values(val):
        if math.isnan(val):
            return 'NaN'
        else:
            return str(int(val))

    # Add rows to tables
    for i, row in enumerate(survey_data):
        # NRS table
        nrs_null = all([math.isnan(v) for v in row[0:4]])
        if not nrs_null:
            survey_definitions.nrs_survey_table.add_row(
                pain_intensity_rating=nrs_values(row[0]),
                relative_pain_intensity_rating=nrs_values(row[1]),
                pain_unpleasantness=nrs_values(row[2]),
                pain_relief_rating=nrs_values(row[3]),
                unix_timestamp=survey_dt[i]
            )

        # VAS table
        vas_null = all([math.isnan(v) for v in row[4:8]])
        if not vas_null:
            survey_definitions.vas_survey_table.add_row(
                pain_intensity_rating=nrs_values(row[4]),
                pain_relief_rating=nrs_values(row[5]),
                relative_pain_intensity_rating=nrs_values(row[6]),
                pain_unpleasantness=nrs_values(row[7]),
                unix_timestamp=survey_dt[i]
            )

        # MPQ table
        mpq_null = all([math.isnan(v) for v in row[9:]])
        if not mpq_null:
            survey_definitions.mpq_survey_table.add_row(
                throbbing=mpq_values(row[9]),
                shooting=mpq_values(row[10]),
                stabbing=mpq_values(row[11]),
                sharp=mpq_values(row[12]),
                cramping=mpq_values(row[13]),
                gnawing=mpq_values(row[14]),
                hot_burning=mpq_values(row[15]),
                aching=mpq_values(row[16]),
                heavy=mpq_values(row[17]),
                tender=mpq_values(row[18]),
                splitting=mpq_values(row[19]),
                tiring_exhausting=mpq_values(row[20]),
                sickening=mpq_values(row[21]),
                fearful=mpq_values(row[22]),
                cruel_punishing=mpq_values(row[23]),
                unix_timestamp=survey_dt[i]
            )

    # Create behavioral processing module, if not existent
    if 'behavior' not in nwbfile.processing:
        nwbfile.create_processing_module(
            name='behavior',
            description='behavioral data'
        )

    # Add survey tables to behavioral processing module
    nwbfile.processing['behavior'].add(survey_definitions.nrs_survey_table)
    nwbfile.processing['behavior'].add(survey_definitions.vas_survey_table)
    nwbfile.processing['behavior'].add(survey_definitions.mpq_survey_table)

    print('Survey data added successfully!')



import os

import numpy as np

from scipy.io import loadmat

from .. import utils


__all__ = ['load_silence_time',
           'compute_baseline',
           'zscore']

def load_silence_time(nwb):
    start = nwb.epochs['start_time'][0]
    stop = nwb.epochs['stop_time'][0]
    return np.array([start, stop])


def compute_baseline(data, tt_data, baseline_time):
    data_time = utils.is_in(tt_data, baseline_time)
    return data[..., data_time]


def zscore(data, axis=-1, mode=None, sampling_freq=None, bad_times=None,
           align_window=None, event_times=None, silence_time=None):

    if mode is None:
        mode = 'silence'

    if mode == 'whole':
        baseline = data
    elif mode == 'between_data':
        tt_data = np.arange(data.shape[axis]) / sampling_freq
        data_start = event_times.min() + align_window[0]
        data_stop = event_times.max() + align_window[1]
        data_time = utils.is_in(tt_data, np.array([data_start, data_stop]))
        for bt in bad_times:
            data_time = data_time & ~utils.is_in(tt_data, bt)
        for et in event_times:
            data_time = data_time & ~utils.is_in(tt_data, et + align_window)
        baseline = data[..., data_time]
    elif mode == 'data':
        tt_data = np.arange(data.shape[-1]) / sampling_freq
        data_start = event_times.min() + align_window[0]
        data_stop = event_times.max() + align_window[1]
        data_time = utils.is_in(tt_data, np.array([data_start, data_stop]))
        for bt in bad_times:
            data_time = data_time & ~utils.is_in(tt_data, bt)
        baseline = data[..., data_time]
    elif mode == 'events':
        tt_data = np.arange(data.shape[-1]) / sampling_freq
        data_time = np.zeros_like(tt_data, dtype=bool)
        for et in event_times:
            data_time = data_time | utils.is_in(tt_data, et + align_window)
        for bt in bad_times:
            data_time = data_time & ~utils.is_in(tt_data, bt)
        baseline = data[..., data_time]
    elif mode == 'silence':
        tt_data = np.arange(data.shape[axis]) / sampling_freq
        baseline = compute_baseline(data, tt_data, silence_time)
    elif mode == 'ratio_silence':
        tt_data = np.arange(data.shape[axis]) / sampling_freq
        baseline = compute_baseline(data, tt_data, silence_time)
        means = baseline.mean(axis=axis, keepdims=True)
        stds = baseline.std(axis=axis, keepdims=True)
        data = data / means
        return data, means, stds, baseline
    elif mode == 'none':
        tt_data = np.arange(data.shape[axis]) / sampling_freq
        baseline = load_baseline(kwargs['block_path'], data, tt_data)
        means = baseline.mean(axis=axis, keepdims=True)
        stds = baseline.std(axis=axis, keepdims=True)
        return data, means, stds, baseline
    else:
        raise ValueError('zscore_mode type {} not recognized.'.format(mode))

    means = baseline.mean(axis=axis, keepdims=True)
    stds = baseline.std(axis=axis, keepdims=True)
    data = (data - means) / stds

    return data, means, stds, baseline



from ecogvis.functions.nwb_copy_file import nwb_copy_file
from datetime import datetime
from dateutil.tz import tzlocal
import pynwb
import numpy as np
from numpy.testing import assert_array_equal
import os


def test_nwb_copy_file():
    # Create a nwb file to be copied
    path_here = os.path.dirname(os.path.abspath(__file__))
    path_old_file = os.path.join(path_here, 'old_file.nwb')
    nwbfile_old = create_random_nwbfile()
    with pynwb.NWBHDF5IO(path_old_file, 'w') as io:
        io.write(nwbfile_old)

    path_new_file = os.path.join(path_here, 'new_file.nwb')

    cp_objs = {
        'institution': True,
        'lab': True,
        'session': True,
        'devices': True,
        'electrode_groups': True,
        'electrodes': True,
        'epochs': True,
        'trials': True,
        'subject': True,
        'acquisition': ['raw_data', 'mic'],
        'stimulus': ['stim1', 'stim2'],
        'ecephys': ['LFP', 'high_gamma']
    }
    nwb_copy_file(
        old_file=path_old_file,
        new_file=path_new_file,
        cp_objs=cp_objs,
        save_to_file=True,
    )

    with pynwb.NWBHDF5IO(path_new_file, 'r') as io:
        nwbfile_new = io.read()

        assert nwbfile_new.institution == nwbfile_old.institution
        assert nwbfile_new.session_start_time == nwbfile_old.session_start_time
        assert nwbfile_new.devices is not None
        assert_array_equal(nwbfile_new.electrodes.columns[0][:], nwbfile_old.electrodes.columns[0][:])
        assert nwbfile_new.electrode_groups.keys() == nwbfile_old.electrode_groups.keys()
        for (v1, v2) in zip(nwbfile_new.stimulus.values(), nwbfile_old.stimulus.values()):
            assert_array_equal(v1.data, v2.data)
        for k in nwbfile_new.acquisition.keys():
            assert_array_equal(nwbfile_new.acquisition[k].data, nwbfile_old.acquisition[k].data)

    # Remove test nwb files
    os.remove(path_old_file)
    os.remove(path_new_file)


def create_random_nwbfile(add_processing=True, add_high_gamma=True):
    """
    Creates a NWB file with fields used by the Chang lab, with random fake data.

    Parameters
    ----------
    add_processing : boolean
        Whether to add processing (LFP) data or not.
    add_high_gamma : boolean
        Whether to add high_gamma data or not.

    Returns
    -------
    nwbfile : nwbfile object
    """
    # Create NWBFile
    start_time = datetime(2017, 4, 3, 11, tzinfo=tzlocal())
    nwbfile = pynwb.NWBFile(session_description='fake data', identifier='NWB123', session_start_time=start_time)

    # Basic fields
    nwbfile.institution = 'My institution'
    nwbfile.lab = 'My lab'
    nwbfile.subject = pynwb.file.Subject(age='5 months', description='description')

    # Add device and electrodes
    device = nwbfile.create_device(name='device')
    electrode_group = nwbfile.create_electrode_group(name='electrode_group0',
                                                     description="an electrode group",
                                                     location="somewhere in the hippocampus",
                                                     device=device)
    n_electrodes = 4
    for idx in np.arange(n_electrodes):
        nwbfile.add_electrode(id=idx,
                              x=1.0, y=2.0, z=3.0,
                              imp=float(-idx),
                              location='CA1', filtering='none',
                              group=electrode_group)

    # Add noise signal as raw data
    electrode_table_region = nwbfile.create_electrode_table_region(list(np.arange(n_electrodes)), 'electrodes_table_region')
    raw_len = 100000
    ephys_data = np.random.rand(raw_len * 4).reshape((raw_len, 4))
    ephys_ts = pynwb.ecephys.ElectricalSeries(
        name='raw_data',
        data=ephys_data,
        electrodes=electrode_table_region,
        starting_time=0.,
        rate=100.
    )
    nwbfile.add_acquisition(ephys_ts)

    # Add noise signal as processing data
    if add_processing:
        ecephys_module = nwbfile.create_processing_module(name='ecephys', description='preprocessed data')
        lfp_len = 10000
        lfp_data = np.random.rand(lfp_len * 4).reshape((lfp_len, 4))
        lfp = pynwb.ecephys.LFP(name='LFP')
        lfp.create_electrical_series(
            name='processed_electrical_series',
            data=lfp_data,
            electrodes=electrode_table_region,
            rate=10.,
            starting_time=0.
        )
        ecephys_module.add_data_interface(lfp)

    # Add noise signal as high gamma
    if add_processing and add_high_gamma:
        hg_data = np.random.rand(lfp_len * 4).reshape((lfp_len, 4))
        hg = pynwb.ecephys.ElectricalSeries(
            name='high_gamma',
            data=hg_data,
            electrodes=electrode_table_region,
            rate=10.,
            description=''
        )
        ecephys_module.add_data_interface(hg)

    # Add noise signals as speaker stimuli and mic recording acquisition
    stim1 = pynwb.TimeSeries(name='stim1', data=np.random.rand(raw_len), starting_time=0.0, rate=100., unit='')
    stim2 = pynwb.TimeSeries(name='stim2', data=np.random.rand(raw_len), starting_time=0.0, rate=100., unit='')
    nwbfile.add_stimulus(stim1)
    nwbfile.add_stimulus(stim2)

    mic = pynwb.TimeSeries(name='mic', data=np.random.rand(raw_len), starting_time=0.0, rate=100., unit='')
    nwbfile.add_acquisition(mic)

    return nwbfile



import numpy as np
from scipy import signal as sgn
from pynwb import NWBHDF5IO, ProcessingModule
from pynwb.ecephys import ElectricalSeries
from ndx_spectrum import Spectrum


def psd_estimate(src_file, type):
    """
    Estimates Power Spectral Density from signals.

    Parameters
    ----------
    src_file : str or path
        Full path to the current NWB file.
    type : str
        ElectricalSeries source. 'raw' or 'preprocessed'.
    """

    # Open file
    with NWBHDF5IO(src_file, mode='r+', load_namespaces=True) as io:
        nwb = io.read()

        # Source ElectricalSeries
        if type == 'raw':
            # Check if there is ElectricalSeries in acquisition group
            for i in list(nwb.acquisition.keys()):
                if isinstance(nwb.acquisition[i], ElectricalSeries):
                    data_obj = nwb.acquisition[i]
        elif type == 'preprocessed':
            data_obj = nwb.processing['ecephys'].data_interfaces['LFP'].electrical_series['preprocessed']

        nChannels = data_obj.data.shape[1]
        nSamples = data_obj.data.shape[0]
        fs = data_obj.rate
        # Welch - window length as power of 2 and keeps dF~0.05 Hz
        dF = .05            # Frequency bin size
        win_len_welch = 2**(np.ceil(np.log2(fs / dF)).astype('int'))   # dF = fs/nfft
        # FFT - using a power of 2 number of samples improves performance
        nfft = int(2**(np.floor(np.log2(nSamples)).astype('int')))
        fx_lim = 200.
        for ch in np.arange(nChannels):  # Iterate over channels
            trace = data_obj.data[:, ch]
            fx_w, py_w = sgn.welch(trace, fs=fs, nperseg=win_len_welch)
            fx_f, py_f = sgn.periodogram(trace, fs=fs, nfft=nfft)
            # saves PSD up to 200 Hz
            py_w = py_w[fx_w < fx_lim]
            fx_w = fx_w[fx_w < fx_lim]
            py_f = py_f[fx_f < fx_lim]
            fx_f = fx_f[fx_f < fx_lim]
            if ch == 0:
                PY_welch = py_w.reshape(-1, 1)
                PY_fft = py_f.reshape(-1, 1)
            else:
                PY_welch = np.append(PY_welch, py_w.reshape(-1, 1), axis=1)
                PY_fft = np.append(PY_fft, py_f.reshape(-1, 1), axis=1)

        # vElectrodes
        elecs_region = nwb.electrodes.create_region(name='electrodes',
                                                    region=np.arange(nChannels).tolist(),
                                                    description='all electrodes')

        # vPSD shape: ('frequency', 'channel')
        spectrum_module_welch = Spectrum(name='Spectrum_welch_' + type,
                                         frequencies=fx_w,
                                         power=PY_welch,
                                         source_timeseries=data_obj,
                                         electrodes=elecs_region)

        spectrum_module_fft = Spectrum(name='Spectrum_fft_' + type,
                                       frequencies=fx_f,
                                       power=PY_fft,
                                       source_timeseries=data_obj,
                                       electrodes=elecs_region)

        # Processing module
        try:      # if ecephys module already exists
            ecephys_module = nwb.processing['ecephys']
        except:   # creates ecephys ProcessingModule
            ecephys_module = ProcessingModule(name='ecephys',
                                              description='Extracellular electrophysiology data.')
            # Add module to NWB file
            nwb.add_processing_module(ecephys_module)
            print('Created ecephys')
        ecephys_module.add_data_interface(spectrum_module_welch)
        ecephys_module.add_data_interface(spectrum_module_fft)

        io.write(nwb)
        print('Spectrum_welch_' + type + ' added to file.')
        print('Spectrum_fft_' + type + ' added to file.')
